// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <Hello.h>

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // Get "hello" object
    //
    CORBA::Object_var obj = orb -> string_to_object("relfile:/Hello.ref");
    if(CORBA::is_nil(obj))
    {
	fprintf(stderr, "%s: cannot read IOR from Hello.ref\n", argv[0]);
	return EXIT_FAILURE;
    }
    
    Hello_var hello = Hello::_narrow(obj);
    assert(!CORBA::is_nil(hello));
    
    //
    // Main loop
    //
    puts("Enter 'h' for hello, 's' to terminate server, or 'x' for exit:");
    int c = 0;
    do
    {
	if(!isspace(c))
            printf("> ");
	c = fgetc(stdin);
        if(c == 'h')
    	    hello -> say_hello();
        else if(c == 's')
    	    hello -> shutdown();
    }
    while(c != EOF && c != 'x');

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
