// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <Hello_impl.h>

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);
    
    //
    // Get a reference to the POA manager
    //
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();
    
    //
    // Create implementation object
    //
    Hello_impl* helloImpl = new Hello_impl(orb, rootPOA);
    PortableServer::ServantBase_var servant = helloImpl;
    Hello_var hello = helloImpl -> _this();
    
    //
    // Run implementation
    //
    manager -> activate();

    //
    // Main loop
    //
    puts("Enter 'h' for hello, or 'x' for exit:");
    int c = 0;
    do
    {
	if(!isspace(c))
	    printf("> ");
	c = fgetc(stdin);
        if(c == 'h')
            hello -> say_hello();
    }
    while(c != EOF && c != 'x');

    orb -> shutdown(true);

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
