// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>
#include <OBE/Net.h>

#include <Conf.h>

#include <stdio.h>

using namespace OB;

#ifdef WIN32
#   include <sys/timeb.h>
#endif

timeval
TimeNow()
{
    timeval tv;
#ifdef WIN32
    struct _timeb timebuffer;
    _ftime(&timebuffer);
    tv.tv_sec = timebuffer.time;
    tv.tv_usec = timebuffer.millitm * 1000;
#else
    gettimeofday(&tv, 0);
#endif
    return tv;
}

timeval
TimeSub(const timeval& a, const timeval& b)
{
    timeval tv;

    tv.tv_sec = a.tv_sec - b.tv_sec;
    tv.tv_usec = a.tv_usec - b.tv_usec;

    tv.tv_sec += tv.tv_usec / 1000000;
    tv.tv_usec = tv.tv_usec % 1000000;

    if(tv.tv_usec < 0)
    {
        tv.tv_usec += 1000000;
        tv.tv_sec -= 1;
    }

    return tv;
}

static char Reply[] =
{
    'G',   'I',   'O',   'P', 001, 000, 001, 001,  '\f',  '\0',  '\0',  '\0',
    '\0',  '\0',  '\0',  '\0', '\0',  '\0',  '\0',  '\0',  '\0',  '\0',
    '\0',  '\0'
};

static char Request[] =
{
    0x47, 0x49, 0x4F, 0x50, // 'G' 'I' 'O' 'P'
    0x01, 0x00, // GIOP version '1' '0'
    0x01, // Flags
    0x00, // Message type
    0x28, 0x00, 0x00, 0x00, // Message size

    // RequestHeader
    0x00, 0x00, 0x00, 0x00, // Service context
    0x00, 0x00, 0x00, 0x00, // Request id
    0x01, // response_expected
    0x00, 0x00, 0x00, // padding
    0x09, 0x00, 0x00, 0x00, // object key (size)
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // object key (body)
    0x04, 0x31, 0x00, // Padding
    0x02, 0x00, 0x00, 0x00, // operation name (size)
    0x66, 0x00, // name ('f' '0')
    0x00, 0x00, // Padding
    0x00, 0x00, 0x00, 0x00 // principal
};

void
ClientLoop(int fd, int reps)
{
    char msg[sizeof(Reply)];
    while(reps-- > 0)
    {
	if(send(fd, Request, sizeof(Request), 0) != sizeof(Request))
	{
	    printf("send failed\n");
	    return;
	}
	if(recv(fd, msg, 12, 0) != 12)
	{
	    printf("recv failed\n");
	    return;
	}
	if(recv(fd, msg + 12, sizeof(msg) - 12, 0) != sizeof(msg) - 12)
	{
	    printf("recv failed\n");
	    return;
	}
	if(memcmp(msg, Reply, sizeof(Reply)) != 0)
	{
	    printf("Bad reply\n");
	    return;
	}
    }
}

int
run(int fd, int reps)
{
    int no;
    setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (char*)&no, sizeof(int));

    struct sockaddr_in addr;
    addr.sin_family = AF_INET;
    addr.sin_port = htons(PERF_PORT);
    addr.sin_addr.s_addr = inet_addr("127.0.0.1");

    if(connect(fd, OB_REINTERPRET_CAST(struct sockaddr*, &addr), sizeof(addr))
       < 0)
    {
	printf("connect failed\n");
	exit(1);
    }
    struct timeval tv = TimeNow();
    ClientLoop(fd, reps);
    tv = TimeSub(TimeNow(), tv);
    printf("Total time needed: %f ms\n",
          ((double)tv.tv_sec + (double)tv.tv_usec / 1000000.0) * 1000.0);
    printf("Time per method call: %f ms\n",
           (((double)tv.tv_sec + (double)tv.tv_usec / 1000000.0)
    	   / (double)reps) * 1000.0);

    return EXIT_SUCCESS;
}

int
main(int argc, char** argv)
{
#ifdef WIN32
    //
    // Winsock startup
    //
    WORD wVersionRequested = MAKEWORD(1, 1);
    WSADATA wsaData;
    if(WSAStartup(wVersionRequested, &wsaData) != 0)
    {
	printf("WSAStartup failed\n");
	return EXIT_FAILURE;
    }
#endif

    int reps = 1000;
    if(argc > 1)
	reps = atol(argv[1]);

    if(reps < 100)
    {
	reps = 100;
	printf("Adjusting reps: %d\n", reps);
    }
    printf("Client role. Sending %d reps\n", reps);

    int fd  = CreateSocket();
    int status = run(fd, reps);
    CloseSocket(fd);

    return status;
}
