// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>
#include <OBE/Net.h>

#include <Conf.h>

#include <stdio.h>

//
// Some operating systems don't know socklen_t
//
#ifdef HAVE_SOCKLEN_T
    typedef socklen_t socklen_type;
#else
#   ifdef HAVE_SOCKETS_WITH_SIZE_T
        typedef size_t socklen_type;
#   else
        typedef int socklen_type;
#   endif
#endif

using namespace OB;

static char Reply[] =
{
    'G',   'I',   'O',   'P', 001, 000, 001, 001,  '\f',  '\0',  '\0',  '\0',
    '\0',  '\0',  '\0',  '\0', '\0',  '\0',  '\0',  '\0',  '\0',  '\0',
    '\0',  '\0'
};

static char Request[] =
{
    0x47, 0x49, 0x4F, 0x50, // 'G' 'I' 'O' 'P'
    0x01, 0x00, // GIOP version '1' '0'
    0x01, // Flags
    0x00, // Message type
    0x28, 0x00, 0x00, 0x00, // Message size

    // RequestHeader
    0x00, 0x00, 0x00, 0x00, // Service context
    0x00, 0x00, 0x00, 0x00, // Request id
    0x01, // response_expected
    0x00, 0x00, 0x00, // padding
    0x09, 0x00, 0x00, 0x00, // object key (size)
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // object key (body)
    0x04, 0x31, 0x00, // Padding
    0x02, 0x00, 0x00, 0x00, // operation name (size)
    0x66, 0x00, // name ('f' '0')
    0x00, 0x00, // Padding
    0x00, 0x00, 0x00, 0x00 // principal
};

void
ServerLoop(int fd)
{
    int nmsg = 0;
    char msg[sizeof(Request)];
    char reply[sizeof(Reply)];
    memcpy(reply, Reply, sizeof(Reply));
    while(1)
    {
	//
	// Read header & body
	//
	if(recv(fd, msg, 12, 0) != 12)
	    break;
	if(recv(fd, msg + 12, sizeof(msg) - 12, 0) != sizeof(msg) - 12)
	    break;
	//
	// Check magic number
	//
	if(memcmp(msg, Request, 4) != 0)
	{
	    printf("Bad request\n");
	    break;
	}

	//
	// Determine whether to byte-swap
	//
	bool swap = false;
	if((msg[4] != 0) != OB::Endian)
	   swap = true;

	//
	// Read request id
	//
	CORBA::ULong id = 0;
	memcpy(&id, msg + 16, 4);
	if(swap)
	{
	    id = ((id << 24) | ((id & 0xff00) << 8)
		   | ((id >> 8) & 0xff00) | (id >> 24));
	}

	//
	// Fill the request_id in the reply
	//
	memcpy(reply + 16, &id, 4);
	++nmsg;

	//
	// Send reply
	//
	if(send(fd, reply, sizeof(reply), 0) != sizeof(reply))
	{
	    printf("send failed\n");
	    break;
	}
    }
    printf("Got %d messages\n", nmsg);
}

int
run(int fd)
{
    int no;
    setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (char*)&no, sizeof(int));
    //setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char*)&no, sizeof(int));

    struct sockaddr_in addr;
    addr.sin_family = AF_INET;
    addr.sin_port = htons(PERF_PORT);
    addr.sin_addr.s_addr = htonl(INADDR_ANY);

    if(bind(fd, OB_REINTERPRET_CAST(struct sockaddr*, &addr), sizeof(addr))
       < 0)
    {
	printf("bind\n");
	return EXIT_FAILURE;
    }

    if(listen(fd, 1) < 0)
    {
	printf("listen\n");
	return EXIT_FAILURE;
    }

    while(1)
    {
	memset(&addr, 0, sizeof(addr));
	socklen_type len = sizeof(addr);

	int n = accept(fd, OB_REINTERPRET_CAST(struct sockaddr*, &addr), &len);
	if(n < 0)
	{
	    printf("Accept failed\n");
	    return EXIT_FAILURE;
	}
	
	ServerLoop(n);
	printf("Done\n");
	CloseSocket(n);
    }
    return EXIT_SUCCESS;
}

int
main(int argc, char** argv)
{
#ifdef WIN32
    //
    // Winsock startup
    //
    WORD wVersionRequested = MAKEWORD(1, 1);
    WSADATA wsaData;
    if(WSAStartup(wVersionRequested, &wsaData) != 0)
    {
	printf("WSAStartup failed\n");
	return EXIT_FAILURE;
    }
#endif

    int fd = CreateSocket();
    if(fd < 0)
    {
	printf("Create Socket failed\n");
	return EXIT_FAILURE;
    }
    int status = run(fd);
    CloseSocket(fd);

    return status;
}
