// **********************************************************************
//
// Copyright (c) 1999
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <I_impl.h>

#include <stdlib.h>
#include <errno.h>

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // Resolve the Root POA
    //
    PortableServer::POA_var root = PortableServer::POA::_narrow(
	CORBA::Object_var(orb -> resolve_initial_references("RootPOA")));
    PortableServer::POAManager_var manager = root -> the_POAManager();

    //
    // Create implementation object
    //
    I_impl* impl = new I_impl();
    PortableServer::ServantBase_var servant = impl;
    I_var p = impl -> _this();

    //
    // Save reference
    //
    CORBA::String_var s = orb -> object_to_string(p);

    const char* refFile = "I.ref";
    FILE* f = fopen(refFile, "w");
    if(f == 0)
    {
	fprintf(stderr, "%s: can't open `%s': %s\n",
		argv[0], refFile, strerror(errno));
	return EXIT_FAILURE;
    }

    fputs(s.in(), f);
    fclose(f);

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
