// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>
#include <OBE/OBPolicies.h>

#include <Hello.h>

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // We need to set the retry policy to RETRY_ALWAYS
    //
    CORBA::Any any;
    any <<= OB::RETRY_ALWAYS;
    CORBA::PolicyList pl;
    pl.length(1);
    pl[0] = orb -> create_policy(OB::RETRY_POLICY_ID, any);
    CORBA::Object_var pmObj =
        orb -> resolve_initial_references("ORBPolicyManager");
    CORBA::PolicyManager_var pm = CORBA::PolicyManager::_narrow(pmObj);
    pm -> add_policy_overrides(pl);

    //
    // Get "hello" object
    //
    CORBA::Object_var obj;
    try
    {
        obj = orb -> string_to_object(argv[1]);
    }
    catch(const CORBA::BAD_PARAM&)
    {
	fprintf(stderr, "%s: Invalid Corbaloc URL\n", argv[0]);
	return EXIT_FAILURE;
    }
    
    Hello_var hello = Hello::_narrow(obj);
    assert(!CORBA::is_nil(hello));
    
    //
    // Main loop
    //
    puts("Enter 'h' for hello, 's' to terminate server, or 'x' for exit:");
    int c = 0;
    do
    {
	if(!isspace(c))
            printf("> ");
	c = fgetc(stdin);
        if(c == 'h')
    	    hello -> say_hello();
        else if(c == 's')
    	    hello -> shutdown();
    }
    while(c != EOF && c != 'x');

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    if(argc != 2)
    {
	fprintf(stderr, "Usage: %s URL\n", argv[0]);
	return EXIT_FAILURE;
    }

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
