// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OBE_BOOTMANAGER_H
#define OBE_BOOTMANAGER_H

#ifndef OBE_IMPORT
#   ifdef OB_DLL
#      define OBE_IMPORT __declspec(dllimport)
#   else
#      define OBE_IMPORT /**/
#   endif
#endif

#include <OBE/BootManager_fwd.h>
#include <OBE/ObjectIdHasher.h>
#include <OBE/Hashtable.h>
#include <OBE/Object.h>

namespace OB
{

class BootManager : virtual public CORBA::Object
{
    BootManager(const BootManager&);
    void operator=(const BootManager&);

    //
    // Set of known bindings
    //
    typedef OB::Hashtable< PortableServer::ObjectId,
    			   CORBA::Object_var,
    			   OBPortableServer::ObjectIdHasher> BootHashTable;
    BootHashTable bindings_; 
    JTCRecursiveMutex bindingsMutex_;

public:

    BootManager();
    ~BootManager();

    //
    // Standard IDL to C++ Mapping
    //
    typedef BootManager_ptr _ptr_type;
    typedef BootManager_var _var_type;

    static inline BootManager_ptr
    _duplicate(BootManager_ptr p)
    {
        if(p)
            p -> _OB_incRef();
        return p;
    }

    static inline BootManager_ptr
    _nil()
    {
        return 0;
    }

    static BootManager_ptr _narrow(CORBA::Object_ptr);

    virtual bool _OB_isLC() const { return true; };

    struct NotFound : public CORBA::UserException
    {
        NotFound() { }

        virtual void _raise() const { throw *this; }
        virtual const char* _rep_id() const;

        virtual CORBA::Exception* _OB_clone() const;
    };

    struct AlreadyExists : public CORBA::UserException
    {
        AlreadyExists() { }

        virtual void _raise() const { throw *this; }
        virtual const char* _rep_id() const;

        virtual CORBA::Exception* _OB_clone() const;
    };

    void add_binding(const PortableServer::ObjectId& oid,
                             CORBA::Object_ptr obj)
	throw(AlreadyExists, CORBA::SystemException);

    void remove_binding(const PortableServer::ObjectId& oid)
        throw(NotFound, CORBA::SystemException);

    //
    // Internal Methods
    //
    RefCountIOR_ptr _OB_locate(const OCI::ObjectKey&);

    void _OB_destroy();
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::BootManager_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(OB::BootManager_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
