// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_IIOP_CONFIG_H
#define OB_IIOP_CONFIG_H

#include <OBE/Basic.h>
#include <OBE/Template.h>
#include <OBE/Object.h>
#include <OBE/OCITypes.h>
#include <OBE/Properties_fwd.h>
#include <OBE/Logger_fwd.h>

namespace OB
{

//
// IIOPConfig converts configuration properties into parameters that
// can be used to create an OCI::Acceptor using an OCI::AccFactory.
// Each invocation of a parse() method may replace parameters created
// by a previous parse() method, therefore callers should invoke
// these methods in the desired order of precedence.
//
class IIOPConfig
{
    //
    // Hide copy-constructor and assignment operator
    //
    IIOPConfig(const IIOPConfig&);
    void operator=(const IIOPConfig&);

    //
    // The properties to be parsed
    //
    Properties_var props_;

    //
    // The Logger
    //
    Logger_var logger_;

    //
    // The parsed parameters
    //
    OCI::ParamSeq params_;

    OCI::Param* findParam(const char*);
    void setParam(const char*, const char*);

public:

    IIOPConfig(Properties_ptr, Logger_ptr);

    //
    // Get the parameters for all properties that have been parsed
    //
    OCI::ParamSeq* getParams();

    //
    // Parse the following property:
    //
    // ooc.iiop.port
    //
    // This property generally only applies to the RootPOAManager.
    //
    void parseDefaultPort();

    //
    // Parse the following property:
    //
    // ooc.iiop.bind
    //
    // This property generally only applies to the RootPOAManager.
    //
    void parseDefaultBind();

    //
    // Parse the following properties:
    //
    // ooc.iiop.backlog
    // ooc.iiop.host
    // ooc.iiop.multi_profile
    // ooc.iiop.numeric
    //
    void parseDefaults();

    //
    // Parse all properties in the standard order of precedence
    //
    void parseAll();
};

} // End of namespace OB

#endif
