// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_INITIAL_SERVICE_MANAGER_H
#define OB_INITIAL_SERVICE_MANAGER_H

#include <OBE/InitialServiceManager_fwd.h>

#include <OBE/NativeTypes.h>
#include <OBE/MarshalStubImpl.h>
#include <OBE/OBORB.h>
#include <OBE/Hashtable.h>

namespace OB
{

//
// The InitialServiceManager class
//
class InitialServiceManager : public RefCount, public JTCMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    InitialServiceManager(const InitialServiceManager&);
    void operator=(const InitialServiceManager&);

    //
    // Set of available initial services
    //
    struct Service
    {
        CORBA::String_var ref;
        CORBA::Object_var obj;
    };

    typedef Hashtable<CORBA::String_var, Service,StringHasher> ServiceMap;
    ServiceMap* services_;

    CORBA::String_var defaultInitRef_;

    bool destroy_; // True if destroy() was called

    ORBInstance_var orbInstance_; // The ORBInstance object

public:

    InitialServiceManager(ORBInstance_ptr);
    virtual ~InitialServiceManager();

    static inline InitialServiceManager_ptr
    _duplicate(InitialServiceManager_ptr p)
    { if(p) p -> _OB_incRef(); return p; }

    static inline InitialServiceManager_ptr _nil()
    { return 0; }

    //
    // Destroy the InitialServiceManager
    //
    void destroy();

    CORBA::Object_ptr resolveInitialReferences(const char*)
	throw(CORBA::SystemException, CORBA::ORB::InvalidName);

    void addInitialReference(const char*, CORBA::Object_ptr)
	throw(CORBA::SystemException, CORBA::ORB::InvalidName);

    void addInitialReference(const char*, CORBA::Object_ptr, bool)
	throw(CORBA::SystemException, CORBA::ORB::InvalidName);

    void addInitialReference(const char*, const char*, bool)
	throw(CORBA::SystemException, CORBA::ORB::InvalidName);
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::InitialServiceManager_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::InitialServiceManager_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
