// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_MARSHAL_STUB_IMPL_H
#define OB_MARSHAL_STUB_IMPL_H

#include <OBE/MarshalStubImpl_fwd.h>
#include <OBE/Client_fwd.h>

#include <OBE/Object.h>
#include <OBE/list.h>

namespace OB
{

//
// A sequence of clients
// 
typedef ObjSeq< Client, int > ClientSeq;
typedef SeqVar< ClientSeq > ClientSeq_var;

//
// A sequence of profiles
// 
typedef ObjSeq< RefCountProfileInfo, int > RefCountProfileInfoSeq;
typedef SeqVar< RefCountProfileInfoSeq > RefCountProfileInfoSeq_var;

//
// The MarshalStubImpl class
//
class MarshalStubImpl : public SimpleRefCount
{
    //
    // Hide copy-constructor and assignment operator
    //
    MarshalStubImpl(const MarshalStubImpl&);
    void operator=(const MarshalStubImpl&);

    //
    // The IOR and the original IOR
    //
    RefCountIOR_var IOR_;
    RefCountIOR_var origIOR_;

    //
    // All client/profile pairs
    //
    OB_STL::list<ClientProfilePair_var>* clientProfilePairs_;

protected:

    //
    // The ORBInstance object
    //
    ORBInstance_var _ob_orbInstance_;

    //
    // The list of policies
    //
    RefCountPolicyList_var _ob_policies_;

    //
    // Get a client/profile pair
    //
    void _OB_getClientProfilePair_nodup(Client_ptr&,
					RefCountProfileInfo_ptr&);

    //
    // Handle a FailureException
    //
    void _OB_handleFailureException(DowncallImpl&, const FailureException&);

public:

    MarshalStubImpl();
    virtual ~MarshalStubImpl();

    static inline MarshalStubImpl_ptr _duplicate(MarshalStubImpl_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline MarshalStubImpl_ptr _nil()
    { return 0; }

    virtual CORBA::Boolean _is_a(const char*);
    virtual CORBA::Boolean _non_existent();
 
    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_setup(ORBInstance_ptr, RefCountIOR_ptr, RefCountIOR_ptr,
                   RefCountPolicyList_ptr);
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::MarshalStubImpl_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::MarshalStubImpl_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
