// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OAINTERFACE_H
#define OB_OAINTERFACE_H

#include <OBE/OAInterface_fwd.h>

#include <OBE/Object.h>
#include <OBE/OCITypes.h>

namespace OB
{

//
// The OAInterface class
//
class OAInterface : public OB::RefCount
{
public:
    
    virtual ~OAInterface() { }

    static inline OAInterface_ptr _duplicate(OAInterface_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline OAInterface_ptr _nil()
    { return 0; }

    //
    // Setup can upcall, so that it can be invoked
    //
    virtual bool setupUpcall(UpcallImpl*)
	throw() = 0;

    //
    // Determine if an object with the provided object key exists
    //
    enum LocateStatus
    {
        UNKNOWN_OBJECT = 0,
        OBJECT_HERE = 1,
        OBJECT_FORWARD = 2
    };
    virtual LocateStatus findByKey(const OCI::ObjectKey&, RefCountIOR_out)
	throw() = 0;

    //
    // Get all profiles that are usable with this OAInterface
    //
    virtual OCI::ProfileInfoSeq* getUsableProfiles(const OCI::IOR&,
						 const CORBA::PolicyList&)
	throw() = 0;

    //
    // Allow associated POAs to receive requests
    //
    virtual void activate()
	throw() = 0;
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::OAInterface_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::OAInterface_ptr p)
{
    return p == 0;
}

} // End namespace CORBA

#endif
