// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OBORB_H
#define OB_OBORB_H

#ifndef OBE_IMPORT
#   ifdef OB_DLL
#      define OBE_IMPORT __declspec(dllimport)
#   else
#      define OBE_IMPORT /**/
#   endif
#endif

#include <OBE/ORBInstance_fwd.h>
#include <OBE/ORBControl_fwd.h>
#include <OBE/Properties_fwd.h>
#include <OBE/Logger_fwd.h>
#ifndef HAVE_OBE_NO_TRACE
#  include <OBE/CoreTraceLevels_fwd.h>
#endif
#include <OBE/Policy_fwd.h>
#include <OBE/OBPolicies_fwd.h>
#include <OBE/OBORB_fwd.h>

#include <OBE/Hashtable.h>
#include <OBE/Any.h>
#include <OBE/OCITypes.h>
#include <OBE/ORB_init.h>

namespace OB
{   

//
// PropertySet definition
//
typedef Properties* PropertySet;
    
//
// For the friend declaration
//
ORBInstance_ptr GetORBInstance(CORBA::ORB_ptr);
        
} // End of namespace CORBA

//
// IDL:ooc.com/CORBA:1.0
//
namespace CORBA
{

//
// IDL:ooc.com/CORBA/ORB:2.3
//
// Locality constrained
//
class ORB : public CORBA::Object, public JTCRecursiveMutex
{
    //
    // Hide copy constructor and operator
    //
    ORB(const ORB&);
    void operator=(const ORB&);

    static OBE_IMPORT const char* ids_[];

    //
    // All ORBs that have been initialized with ORB_init()
    //
    // Note: Do not translate this to Java. For the Java mapping,
    // ORB_init() always returns a new ORB instance.
    //
    typedef OB::Hashtable<CORBA::String_var, ORB_var, 
	OB::StringHasher> ORBImplMap;
    static ORBImplMap orbImplMap_;
    static JTCRecursiveMutex orbImplMapMutex_;

    //
    // The ORB id
    //
    CORBA::String_var id_;

    //
    // The ORB Control
    //
    OB::ORBControl_var orbControl_;

    //
    // Default set of policies
    //
    CORBA::PolicyList policies_;
    void initializeDefaultPolicies();

protected:

    //
    // The ORBInstance object
    //
    OB::ORBInstance_var orbInstance_;

    //
    // Has the ORB been destroyed?
    //
    bool destroy_;

    //
    // Private constructor and destructor
    //
    ORB(int ac, char** av, const char*,
#ifndef HAVE_OBE_NO_TRACE
	OB::CoreTraceLevels_ptr,
#endif // !HAVE_OBE_NO_TRACE
	OB::Properties_ptr, OB::Logger_ptr);
    ~ORB();

    //
    // ORB_init needs access to orbImplMap_
    //
    friend OBCORBA::ORB_ptr
    OBCORBA::ORB_init(int&, char**, OB::Properties_ptr, OB::Logger_ptr,
		      const char*, const char*);

    //
    // GetORBInstance() needs access to orbInstance_
    //
    friend OB::ORBInstance_ptr OB::GetORBInstance(CORBA::ORB_ptr);

public:

    typedef ORB_ptr _ptr_type;
    typedef ORB_var _var_type;

    static inline ORB_ptr
    _duplicate(ORB_ptr p)
    {
        if(p)
            p -> _OB_incRef();
        return p;
    }

    static inline ORB_ptr
    _nil()
    {
        return 0;
    }

    static ORB_ptr _narrow(CORBA::Object_ptr);

    static const char** _OB_staticIds();

    virtual bool _OB_isLC() const { return true; };

    //
    // IDL:omg.org/CORBA/ORB/ObjectId:1.0
    //
    typedef char* ObjectId;
    typedef CORBA::String_var ObjectId_var;
    typedef CORBA::String_out ObjectId_out;

    //
    // IDL:omg.org/CORBA/ORB/ObjectIdList:1.0
    //
    class OBUnique_ObjectIdList { };

    typedef OB::StrSeq< OBUnique_ObjectIdList > ObjectIdList;
    typedef OB::SeqVar< OB::StrSeq< OBUnique_ObjectIdList > > ObjectIdList_var;
    typedef OB::SeqOut< OB::StrSeq< OBUnique_ObjectIdList > > ObjectIdList_out;

    //
    // IDL:omg.org/CORBA/ORB/InvalidName:1.0
    //
    struct InvalidName : public CORBA::UserException
    {
        InvalidName() { }

        virtual void _raise() const { throw *this; }
        virtual const char* _rep_id() const;

        virtual CORBA::Exception* _OB_clone() const;
    };

    //
    // IDL:omg.org/CORBA/ORB/object_to_string:1.0
    //
    char* object_to_string(CORBA::Object_ptr obj);

    //
    // IDL:omg.org/CORBA/ORB/string_to_object:1.0
    //
    CORBA::Object_ptr string_to_object(const char* str);

    //
    // IDL:omg.org/CORBA/ORB/resolve_initial_references:1.0
    //
    CORBA::Object_ptr resolve_initial_references(const char* identifier);

    //
    // IDL:omg.org/CORBA/ORB/register_initial_reference:1.0
    //
    void register_initial_reference(const char* id, CORBA::Object_ptr obj);

    //
    // IDL:omg.org/CORBA/ORB/run:1.0
    //
    void run();

    //
    // IDL:omg.org/CORBA/ORB/shutdown:1.0
    //
    void shutdown(CORBA::Boolean wait_for_completion);

    //
    // IDL:omg.org/CORBA/ORB/destroy:1.0
    //
    void destroy();

    //
    // IDL:omg.org/CORBA/ORB/create_policy:1.0
    //
    Policy_ptr create_policy(PolicyType type, const CORBA::Any& val);
};

} // End of namespace CORBA

//
// IDL:ooc.com/OBCORBA:1.0
//
namespace OBCORBA
{

//
// IDL:ooc.com/OBCORBA/ORB:1.0
//
// Locality constrained
//
class ORB : public CORBA::ORB
{
    //
    // Hide copy constructor and operator
    //
    ORB(const ORB&);
    void operator=(const ORB&);

    static OBE_IMPORT const char* ids_[];

    //
    // Private constructor and destructor
    //
    ORB(int ac, char** av, const char*,
#ifndef HAVE_OBE_NO_TRACE
	OB::CoreTraceLevels_ptr,
#endif // !HAVE_OBE_NO_TRACE
	OB::Properties_ptr, OB::Logger_ptr);
    ~ORB();

    //
    // ORB_init may create ORBs
    //
    friend OBCORBA::ORB_ptr
    OBCORBA::ORB_init(int&, char**, OB::Properties_ptr, OB::Logger_ptr,
		      const char*, const char*);
public:

    typedef ORB_ptr _ptr_type;
    typedef ORB_var _var_type;

    static inline ORB_ptr
    _duplicate(ORB_ptr p)
    {
        if(p)
            p -> _OB_incRef();
        return p;
    }

    static inline ORB_ptr
    _nil()
    {
        return 0;
    }

    static ORB_ptr _narrow(CORBA::Object_ptr);

    static const char** _OB_staticIds();

    virtual bool _OB_isLC() const { return true; };

    //
    // IDL:ooc.com/OBCORBA/ORB/properties:1.0
    //
    OB::PropertySet properties();

    //
    // IDL:ooc.com/OBCORBA/ORB/logger:1.0
    //
    OB::Logger_ptr logger();

    //
    // IDL:ooc.com/OBCORBA/ORB/create_connection_reuse_policy:1.0
    //
    OB::ConnectionReusePolicy_ptr
    create_connection_reuse_policy(CORBA::Boolean value);

    //
    // IDL:ooc.com/OBCORBA/ORB/create_protocol_policy:1.0
    //
    OB::ProtocolPolicy_ptr
    create_protocol_policy(OCI::ProtocolId value);

    //
    // IDL:ooc.com/OBCORBA/ORB/create_retry_policy:1.0
    //
    OB::RetryPolicy_ptr
    create_retry_policy(CORBA::Short value);

    //
    // IDL:ooc.com/OBCORBA/ORB/create_timeout_policy:1.0
    //
    OB::TimeoutPolicy_ptr
    create_timeout_policy(CORBA::ULong value);

    //
    // IDL:ooc.com/OBCORBA/ORB/create_connection_timeout_policy:1.0
    //
    OB::ConnectTimeoutPolicy_ptr
    create_connection_timeout_policy(CORBA::ULong value);

    //
    // IDL:ooc.com/OBCORBA/ORB/create_request_timeout_policy:1.0
    //
    OB::RequestTimeoutPolicy_ptr
    create_request_timeout_policy(CORBA::ULong value);
};

} // End of namespace OBCORBA

//
// IDL:omg.org/CORBA/ORB:2.3
//
namespace CORBA
{

inline void
release(CORBA::ORB_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(CORBA::ORB_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

//
// IDL:ooc.com/OBCORBA/ORB:1.0
//
namespace CORBA
{

inline void
release(OBCORBA::ORB_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(OBCORBA::ORB_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
