// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_ORB_CONTROL_H
#define OB_ORB_CONTROL_H

#include <OBE/ORBControl_fwd.h>
#include <OBE/ORBInstance_fwd.h>
#include <OBE/Client.h>

namespace OB
{

//
// The ORBControl class
//
class ORBControl : public RefCount, public JTCRecursiveMutex
{
protected:

    //
    // The ORB Instance
    //
    OB::ORBInstance_var orbInstance_;

    //
    // The ORBControl state
    //
    enum State
    {
	StateNotRunning,
	StateRunning,
	StateServerShutdown,
	StateClientShutdown,
	StateDestroyed
    };
    State state_;

    //
    // Has shutdown been called?
    //
    bool shutdown_;

    //
    // The thread id of the main thread (that is the first thread that
    // calls run, perform_work or work_pending.
    //
    JTCThreadId mainThread_;

    //
    // Complete shutdown of the ORB, if necessary. Return true if the
    // shutdown is complete.
    //
    virtual void completeServerShutdown()
	throw() = 0;

    //
    // Validate the state
    //
    virtual void validateState()
	throw(CORBA::SystemException) = 0;

    //
    // Are we inside a method invocation?
    //
    virtual bool insideMethodInvocation()
	throw() = 0;

    virtual void blockServerShutdownComplete()
	throw() = 0;

    ORBControl(ORBInstance_ptr);
    virtual ~ORBControl();

public:

    static inline ORBControl_ptr _duplicate(ORBControl_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline ORBControl_ptr _nil()
    { return 0; }
    
    //
    // Destroy the ObjectFactory
    //
    void destroy()
	throw(CORBA::SystemException);

    //
    // Initialize the root POA
    //
    virtual void initializeRootPOA() = 0;

    //
    // Run the ORB event loop
    //
    virtual void run()
	throw(CORBA::SystemException) = 0;

    //
    // Shutdown the server side of the ORB
    //
    virtual void shutdownServer(bool)
	throw(CORBA::SystemException) = 0;

    //
    // Shutdown the server (if necessary) & client side of the ORB
    //
    virtual void shutdownServerClient()
	throw(CORBA::SystemException) = 0;
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::ORBControl_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::ORBControl_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
