// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OBE_POAMANAGER_H
#define OBE_POAMANAGER_H

#ifndef OBE_IMPORT
#   ifdef OB_DLL
#      define OBE_IMPORT __declspec(dllimport)
#   else
#      define OBE_IMPORT /**/
#   endif
#endif

#include <OBE/POAManager_fwd.h>
#include <OBE/Server_fwd.h>
#include <OBE/POA_fwd.h>

#include <OBE/OAInterface_fwd.h>
#include <OBE/ORBInstance_fwd.h>
#include <OBE/OCI.h>
#include <OBE/OBPOAManager.h>

#ifndef HAVE_OBE_NO_COLLOC
#  include <OBE/CollocatedServer_fwd.h>
#endif // !HAVE_OBE_NO_COLLOC

namespace OB
{

//
// Forward declarations
// 
struct ObjectKeyData;

} // End of namespace OB

namespace PortableServer
{

class POAManager : public CORBA::Object, public JTCRecursiveMutex
{
    POAManager(const POAManager&);
    void operator=(const POAManager&);

    //
    // The ORBInstance
    //
    OB::ORBInstance_var orbInstance_;

    //
    // Set of connected POAs
    //
    PortableServer::POA_var poa_;

    //
    // The POAManager state
    //
    enum State
    {
	HOLDING, ACTIVE, INACTIVE
    };
    State state_;

    //
    // The OCI acceptors
    //
    OCI::AcceptorSeq acceptors_;

    //
    // The server manager
    //
    OB::ServerManager_var serverManager_;

    //
    // The OAInterface
    //
    OB::OAInterface_var oaInterface_;

public:

    POAManager(OB::ORBInstance_ptr)
	throw(CORBA::SystemException);
    virtual ~POAManager();

    typedef POAManager_ptr _ptr_type;
    typedef POAManager_var _var_type;

    static inline POAManager_ptr
    _duplicate(POAManager_ptr p)
    {
        if(p)
            p -> _OB_incRef();
        return p;
    }

    static inline POAManager_ptr
    _nil()
    {
        return 0;
    }

    static POAManager_ptr _narrow(CORBA::Object_ptr);

    virtual bool _OB_isLC() const { return true; };

    //
    // Standard IDL to C++ Mapping
    //
    void activate()
        throw(CORBA::SystemException);
    
    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //

    //
    // Create the root POA
    //
    void _OB_createRootPOA()
	throw();

    //
    // Locate the POA for this object key. If the POA cannot be
    // located a _nil POA is returned.
    //
    PortableServer::POA_ptr _OB_POA_nodup()
        throw(CORBA::SystemException);

#ifndef HAVE_OBE_NO_COLLOC
    //
    // Retrieve the CollocatedServer for this POAManager
    //
    OB::CollocatedServer_ptr _OB_getCollocatedServer()
	throw();
#endif // !HAVE_OBE_NO_COLLOC

    //
    // Deactivate the POAManager
    //
    void _OB_deactivate()
	throw(CORBA::SystemException);
 
    //
    // Destroy the POAManager
    //
    void _OB_destroy()
	throw(CORBA::SystemException);

    //
    // Get the OCI acceptors
    //
    OCI::AcceptorSeq*
    _OB_getAcceptors()
	throw(CORBA::SystemException);

};

} // End of namespace PortableServer


namespace CORBA
{

inline void
release(PortableServer::POAManager_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(PortableServer::POAManager_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
