// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_SERVANT_BASE_H
#define OB_SERVANT_BASE_H

#include <OBE/POAServantBase_fwd.h>
#include <OBE/Object_fwd.h>
#include <OBE/POA_fwd.h>
#include <OBE/OBPOAManager_fwd.h>
#include <OBE/Basic.h>
#include <OBE/Except.h>

namespace CORBA
{

//
// RepositoryId
//
typedef char* RepositoryId;

} // End of namespace CORBA

namespace PortableServer
{

//
// ObjectId
//  
class OBUnique_ObjectId;
typedef OB::FixSeq< CORBA::Octet, OBUnique_ObjectId > ObjectId;
typedef OB::SeqVar< OB::FixSeq< CORBA::Octet, OBUnique_ObjectId > >
    ObjectId_var;

//
// Conversion between ObjectId and string/wstring
//
char* ObjectId_to_string(const ObjectId&);

ObjectId* string_to_ObjectId(const char*);

//
// The PortableServer::ServantBase class, as defined by the C++
// mapping
//
class ServantBase
{

protected:

    //
    // Dispatch operations of the base interface
    //
    void _OB_dispatchBase(OB::UpcallImpl*);

    //
    // Find an operation
    //
    static CORBA::Long _OB_findOperation(OB::UpcallImpl*, const char*[],
					 CORBA::ULong);

    ServantBase() { }
    ServantBase(const ServantBase&) { }
    ServantBase& operator=(const ServantBase&) { return *this; }
    
public:
    
    virtual ~ServantBase() { }
    
    virtual POA_ptr _default_POA();
    
    virtual CORBA::Boolean _is_a(const char* logical_type_id)
        throw(CORBA::SystemException);
        
    virtual CORBA::Boolean _non_existent()
        throw(CORBA::SystemException);

    virtual CORBA::RepositoryId _primary_interface(const ObjectId&,
						   POA_ptr) = 0;
        
    virtual void _add_ref() { }
    virtual void _remove_ref() { }
    
    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    virtual void _OB_dispatch(OB::UpcallImpl*) = 0;

    CORBA::Object_ptr _OB_createReference();
    virtual CORBA::ULong _OB_getRefCount();

    virtual bool _OB_hasNativeTypes();
};

//
// The PortableServer::RefCountServantBase class, as defined by the
// C++ mapping.
//
class RefCountServantBase : public virtual ServantBase,
                            public virtual OB::RefCount
{
protected:
    
    RefCountServantBase() { }
    RefCountServantBase(const RefCountServantBase&) { }
    RefCountServantBase&
    operator=(const RefCountServantBase&) { return *this; }

public:
    
    virtual void _add_ref();
    virtual void _remove_ref();
    virtual CORBA::ULong _OB_getRefCount();
};

} // End of namespace PortableServer

#endif
