// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_SERVER_H
#define OB_SERVER_H

#include <OBE/Server_fwd.h>
#include <OBE/OAInterface_fwd.h>
#include <OBE/ORBInstance_fwd.h>
#ifndef HAVE_OBE_NO_COLLOC
#include <OBE/CollocatedServer_fwd.h>
#endif // !HAVE_OBE_NO_COLLOC

#include <OBE/OCI.h>
#include <OBE/POAManager_fwd.h> // For the friend declaration

namespace OB
{

//
// The Server class
//
class Server : virtual public RefCount
{
    //
    // Hide copy-constructor and assignment operator
    //
    Server(const Server&);
    void operator=(const Server&);

protected:
    Server();
    virtual ~Server();

public:
    
    static inline Server_ptr _duplicate(Server_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline Server_ptr _nil()
    { return 0; }

    //
    // Destroy the server
    //
    virtual void destroy() = 0;

    //
    // Dispatch any requests that arrive for the Server
    //
    virtual void activate() = 0;
};

//
// A sequence of servers
// 
typedef ObjSeq< Server, int > ServerSeq;
typedef SeqVar< ServerSeq > ServerSeq_var;

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::Server_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::Server_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

namespace OB
{

//
// The ServerManager class
//
class ServerManager : public RefCount, public JTCMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    ServerManager(const ServerManager&);
    void operator=(const ServerManager&);

    bool destroy_; // True is destroy() was called
#ifndef HAVE_OBE_NO_COLLOC
    CollocatedServer_var collocatedServer_; // The collocated server
#endif // !HAVE_OBE_NO_COLLOC
    ServerSeq allServers_; // All servers

    ServerManager(ORBInstance_ptr, const OCI::AcceptorSeq&, OAInterface_ptr);
    ~ServerManager();

    //
    // POAManager creates ServerManager
    //
    friend class ::PortableServer::POAManager;

public:

    static inline ServerManager_ptr _duplicate(ServerManager_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline ServerManager_ptr _nil()
    { return 0; }

    //
    // Destroy the server manager, including all servers it contains
    //
    void destroy();

    //
    // Dispatch all requests for each server
    //
    void activate();

#ifndef HAVE_OBE_NO_COLLOC
    //
    // Get the collocated server
    //
    CollocatedServer_ptr getCollocatedServer();
#endif // !HAVE_OBE_NO_COLLOC
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::ServerManager_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::ServerManager_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
