// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/RefCounted.h>
#include <OBE/BootManager.h>

using namespace CORBA;
using namespace OB;

void
OBDuplicate(OB::BootManager_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(OB::BootManager_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

OB::BootManager_ptr
OB::BootManager::_narrow(CORBA::Object_ptr p)
{
    if(!CORBA::is_nil(p))
    {
        BootManager_ptr v = dynamic_cast<BootManager_ptr>(p);
        if(v)
            return _duplicate(v);
    }

    return _nil();
}

const char*
OB::BootManager::NotFound::_rep_id() const
{
    return "IDL:ooc.com/OB/BootManager/NotFound:1.0";
}

CORBA::Exception*
OB::BootManager::NotFound::_OB_clone() const
{
    return new NotFound(*this);
}

const char*
OB::BootManager::AlreadyExists::_rep_id() const
{
    return "IDL:ooc.com/OB/BootManager/AlreadyExists:1.0";
}

CORBA::Exception*
OB::BootManager::AlreadyExists::_OB_clone() const
{
    return new AlreadyExists(*this);
}


// ----------------------------------------------------------------------
// BootManager constructor/destructor
// ----------------------------------------------------------------------

BootManager::BootManager()
    : bindings_(17)
{
}

BootManager::~BootManager()
{
}

// ----------------------------------------------------------------------
// BootManager Standard IDL to C++ Mapping
// ----------------------------------------------------------------------

void
BootManager::add_binding(const PortableServer::ObjectId& oid,
			      Object_ptr obj)
    throw(BootManager::AlreadyExists, SystemException)
{
    //
    // If binding id is not already mapped add the binding.
    //
    JTCSynchronized sync(bindingsMutex_);

    if(bindings_.containsKey(oid))
        throw AlreadyExists();

    bindings_.put(oid, Object::_duplicate(obj));
}

void
BootManager::remove_binding(const PortableServer::ObjectId& oid)
    throw(BootManager::NotFound, SystemException)
{
    //
    // If binding id is mapped remove the binding
    //
    JTCSynchronized sync(bindingsMutex_);

    if(!bindings_.containsKey(oid))
        throw NotFound();

    bindings_.remove(oid);
}

// ----------------------------------------------------------------------
// BootManager Internal Methods
// ----------------------------------------------------------------------

RefCountIOR_ptr
BootManager::_OB_locate(const OCI::ObjectKey& key)
{
    //
    // Convert the OCI::ObjectKey to a PortableServer::ObjectId
    //
    PortableServer::ObjectId oid(key.length());
    oid.length(key.length());
    memcpy(oid.get_buffer(), key.get_buffer(), key.length());

    //
    // Check the internal hash table to find the binding for the
    // requested ObjectId.
    //
    Object_var obj;

    JTCSynchronized sync(bindingsMutex_);
    bindings_.get(oid, obj);

    if(is_nil(obj))
        return RefCountIOR::_nil();

    return obj -> _OB_IOR();
}

void
BootManager::_OB_destroy()
{
    JTCSynchronized sync(bindingsMutex_);
    bindings_.clear();
}
