// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/Client.h>
#include <OBE/Except.h>
#include <OBE/OCI.h>
#include <OBE/ORBInstance.h>
#include <OBE/OBPolicies.h>
#include <OBE/RefCounted.h>
#include <OBE/OBPOAManager.h>

#include <ClientS.h>
#include <ORBInstanceS.h>
#ifndef HAVE_OBE_NO_COLLOC
#  include <CollocatedClient.h>
#endif // !HAVE_OBE_NO_COLLOC
#include <GIOPClient.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ClientManagerS private member implementation
// ----------------------------------------------------------------------

void
ClientManagerS::getCollocatedPairs(const IOP::IOR& ior,
				   const PolicyList& policies,
				   OB_STL::list<ClientProfilePair_var>* pairs)
{
#ifndef HAVE_OBE_NO_COLLOC
  ULong j;

  //
  // Get the server ORBInstance
  //
  ORBInstanceS* orbInstance = dynamic_cast<ORBInstanceS*>(orbInstance_.in());
  assert(orbInstance);

  //
  // Get the POAManager
  //
  OBPortableServer::POAManager_var manager = orbInstance -> getPOAManager();

  if(!CORBA::is_nil(manager))
    {
      OCI::AcceptorSeq_var acceptors = manager -> get_acceptors();
      bool local = false;
      for(j = 0 ; j < acceptors -> length() && !local ; j++)
	{
	  OCI::ProfileInfoSeq_var localProfileInfos =
	    acceptors[j] -> get_local_profiles(ior);
	  if(localProfileInfos -> length() > 0)
	    local = true;
	}

      if(local)
	{
	  //
	  // Retrieve the CollocatedServer from the POAManager
	  //
	  CollocatedServer_var collocatedServer =
	    manager -> _OB_getCollocatedServer();
		
	  //
	  // Create a new CollocatedClient and add the new client to
	  // both the list of all clients, and the list of clients
	  // that is returned
	  //
	  Client_var client = new CollocatedClient(collocatedServer);

	  allClients_.push_back(client);

	  OCI::ProfileInfoSeq_var profileInfos =
	    client -> getUsableProfiles(ior, policies);
	  for(j = 0 ; j < profileInfos -> length() ; j++)
	    {
	      ClientProfilePair_var pair = new ClientProfilePair();
	      pair -> client = client;
	      pair -> profile =
		new RefCountProfileInfo(profileInfos[j]);
	      pairs -> push_back(pair);
	    }

	  //
	  // TODO: Introduce reusable CollocatedClients?
	  //
	}
    }
#endif // !HAVE_OBE_NO_COLLOC
}
