// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/Basic.h>

#ifndef HAVE_OBE_NO_COLLOC

#include <OBE/Client.h>
#include <OBE/Except.h>
#include <OBE/Stream.h>

#include <CollocatedClient.h>
#include <CollocatedServer.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OB::CollocatedClient_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::CollocatedClient_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// CollocatedClient constructor and destructor
// ----------------------------------------------------------------------

CollocatedClient::CollocatedClient(CollocatedServer_ptr server)
    : Client(),
      server_(CollocatedServer::_duplicate(server))
{
}

CollocatedClient::~CollocatedClient()
{
}

// ----------------------------------------------------------------------
// CollocatedClient public member implementation
// ----------------------------------------------------------------------

void
CollocatedClient::destroy()
{
    // Nothing to do here
}

ULong
CollocatedClient::requestId()
{
    //
    // This operation *must* delegate to CollocatedServer, because
    // request IDs must be unique per CollocatedServer, not per
    // CollocatedClient.
    //
    return server_ -> requestId();
}

OCI::ProfileInfoSeq*
CollocatedClient::getUsableProfiles(const OCI::IOR& ior,
				    const PolicyList& policies)
{
    return server_ -> getUsableProfiles(ior, policies);
}

DowncallEmitter_ptr
CollocatedClient::startDowncall(DowncallImpl* down)
    throw()
{
    return DowncallEmitter::_duplicate(this);
}

bool
CollocatedClient::equal(Client_ptr cl)
{
    CollocatedClient_ptr client = dynamic_cast<CollocatedClient_ptr>(cl);
    if(is_nil(client))
	return false;
    
    if(server_.in() != client -> server_.in())
	return false;
    
    return true;
}

void
CollocatedClient::send(DowncallImpl* down)
    throw()
{
    server_ -> send(down);
}

void
CollocatedClient::sendReceive(DowncallImpl* down)
    throw()
{
    server_ -> sendReceive(down);
}

#endif // !HAVE_OBE_NO_COLLOC
