// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_COLLOCATED_SERVER_H
#define OB_COLLOCATED_SERVER_H

#include <OBE/OAInterface_fwd.h>
#include <OBE/Server.h>

#include <OBE/CollocatedServer_fwd.h>

#include <OBE/Upcall.h>

namespace OB
{

//
// The CollocatedUpcallReturn class
//
class CollocatedUpcallReturn : public UpcallReturn
{
    //
    // Hide copy-constructor and assignment operator
    //
    CollocatedUpcallReturn(const CollocatedUpcallReturn&);
    void operator=(const CollocatedUpcallReturn&);

    //
    // The downcall
    //
    DowncallImpl* down_;

public:

    CollocatedUpcallReturn(DowncallImpl* down) : down_(down) { }

    //
    // Called upon return from an upcall
    //
    virtual void upcallBeginReply(UpcallImpl*);
    virtual void upcallEndReply(UpcallImpl*);
    virtual void upcallUserException(UpcallImpl*,
				     const CORBA::UserException&);
    virtual void upcallSystemException(UpcallImpl*,
				       const CORBA::SystemException&);
    virtual void upcallForward(UpcallImpl*, RefCountIOR_ptr);
};

//
// The CollocatedServer class
//
class CollocatedServer : public Server
{
    //
    // Hide copy-constructor and assignment operator
    //
    CollocatedServer(const CollocatedServer&);
    void operator=(const CollocatedServer&);

    //
    // True if destroy() was called
    //
    bool destroy_;

    //
    // The object adapter interface
    //
    OAInterface_var oaInterface_;

public:
    
    CollocatedServer(OAInterface_ptr);
    virtual ~CollocatedServer();

    static inline CollocatedServer_ptr _duplicate(CollocatedServer_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline CollocatedServer_ptr _nil()
    { return 0; }

    //
    // Destroy the server
    //
    virtual void destroy();

    //
    // Dispatch any requests that arrive for the Server
    //
    virtual void activate();

    //
    // Called to emit downcalls
    //
    virtual void send(DowncallImpl*)
        throw();
    virtual void sendReceive(DowncallImpl*)
        throw();

    //
    // Get a new request ID
    //
    CORBA::ULong requestId();

    //
    // Get the usable profiles
    //
    OCI::ProfileInfoSeq* getUsableProfiles(const OCI::IOR&,
					   const CORBA::PolicyList&);
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::CollocatedServer_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::CollocatedServer_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
