// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_GIOP_CLIENT_WORKER_H
#define OB_GIOP_CLIENT_WORKER_H

#include <OBE/OCIClient_fwd.h>

#include <GIOPClient_fwd.h>
#include <GIOPClientWorker_fwd.h>

#include <OBE/Downcall.h>

namespace OB
{

//
// The GIOPClientWorker class
//
class GIOPClientWorker : public DowncallEmitter, public JTCMutex
{
    //
    // Hide copy-constructor and assignment operator
    //
    GIOPClientWorker(const GIOPClientWorker&);
    void operator=(const GIOPClientWorker&);

protected:

    ORBInstance_var orbInstance_; // The ORBInstance
    GIOPClient_var client_; // The client
    OCI::Transport_var transport_; // The transport
#ifndef HAVE_OBE_NO_TRACE
    bool closeLogged_; // True, if connection closure was already logged
#endif // !HAVE_OBE_NO_TRACE
    CORBA::ULong shutdownTimeout_; // The shutdown timeout

    //
    // The worker's state
    //
public:
    enum State
    {
	StateActive,
	StateClosing,
	StateError,
	StateClosed
    };
protected:
    State state_;
    
    //
    // The failure exception. This must be set if state_ >=
    // StateCloseError.
    //
    CORBA::SystemException* exception_;

#ifndef HAVE_OBE_NO_TRACE
    //
    // Emit a trace message when closing the connection
    //
    void logCloseConnection();
#endif // !HAVE_OBE_NO_TRACE

    //
    // Shut down the connection, graceful and abortive
    //
    void gracefulShutdown();
    void abortiveShutdown();

    //
    // Change the state of the worker due to an exception
    //
    void exception(State, const CORBA::SystemException&);

    //
    // Execute incoming message
    //
    void execute(DowncallImpl*, CORBA::Octet)
	throw();

    //
    // Send a downcall without synchronization
    //
    bool sendNoSync(DowncallImpl*)
	throw();

public:

    GIOPClientWorker(ORBInstance_ptr, GIOPClient_ptr, OCI::Transport_ptr);
    virtual ~GIOPClientWorker();

    static inline GIOPClientWorker_ptr _duplicate(GIOPClientWorker_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline GIOPClientWorker_ptr _nil()
    { return 0; }

    //
    // Destroy the worker
    //
    void destroy();

    //
    // Get the OCI transport
    //
    OCI::Transport_ptr transport();

    //
    // Called to emit downcalls
    //
    virtual void send(DowncallImpl*)
        throw();
    virtual void sendReceive(DowncallImpl*)
        throw();
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::GIOPClientWorker_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::GIOPClientWorker_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
