// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/OAInterface.h>
#include <OBE/Logger.h>
#include <OBE/Except.h>
#include <OBE/OCI.h>
#include <OBE/ORBInstance.h>

#include <GIOPServer.h>
#include <GIOPServerStarter.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// GIOPServer constructor and destructor
// ----------------------------------------------------------------------

GIOPServer::GIOPServer(ORBInstance_ptr orbInstance,
		       OCI::Acceptor_ptr acceptor,
                       OAInterface_ptr oaInterface)
    : Server(),
      orbInstance_(ORBInstance::_duplicate(orbInstance)),
      destroy_(false),
      acceptor_(OCI::Acceptor::_duplicate(acceptor)),
      oaInterface_(OAInterface::_duplicate(oaInterface))
{
    try
    {
        starter_ = new GIOPServerStarter(orbInstance_, acceptor_,
					 oaInterface_);
    }
    catch(...)
    {
	destroy_ = true;
	_OB_setRef(0);
	throw;
    }
}

GIOPServer::~GIOPServer()
{
    assert(destroy_);
    assert(is_nil(starter_));
}

// ----------------------------------------------------------------------
// GIOPServer public member implementation
// ----------------------------------------------------------------------

void
GIOPServer::destroy()
{
    //
    // Don't destroy twice
    //
    if(destroy_)
	return;

    //
    // Set the destroy flag
    //
    destroy_ = true;

    //
    // Close and remove the starter
    //
    assert(!is_nil(starter_));
    starter_ -> setState(GIOPServerStarter::StateClosed);
    starter_ = GIOPServerStarter::_nil();
}

void
GIOPServer::activate()
{
    assert(!destroy_);
    assert(!is_nil(starter_));
    starter_ -> setState(GIOPServerStarter::StateActive);
}
