// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_GIOP_SERVER_STARTER_H
#define OB_GIOP_SERVER_STARTER_H

#include <OBE/OAInterface_fwd.h>
#include <OBE/OCI_fwd.h>
#include <OBE/ORBInstance_fwd.h>
#include <OBE/list.h>

#include <GIOPServerStarter_fwd.h>

#include <GIOPServerWorker.h>

namespace OB
{

//
// The GIOPServerStarter class
//
class GIOPServerStarter : public RefCount, public JTCMonitor
{
    //
    // Hide copy-constructor and assignment operator
    //
    GIOPServerStarter(const GIOPServerStarter&);
    void operator=(const GIOPServerStarter&);

protected:

    ORBInstance_var orbInstance_; // The ORBInstance
    OCI::Acceptor_var acceptor_; // The acceptor
    OAInterface_var oaInterface_; // The object adapter interface
    OB_STL::list<GIOPServerWorker_var> workers_; // The workers

    //
    // The starter's state
    //
public:
    enum State
    {
	StateActive,
	StateHolding,
	StateClosed
    };
protected:
    State state_;

#ifndef HAVE_OBE_NO_TRACE
    //
    // Emit a trace message when closing the acceptor
    //
    void logCloseAcceptor();
#endif // !HAVE_OBE_NO_TRACE

    //
    // The starter thread
    //
    class StarterThread : public JTCThread
    {
	GIOPServerStarter_var starter_;

    public:
	
	StarterThread(JTCThreadGroupHandle& group,
		      GIOPServerStarter_ptr starter)
	    : JTCThread(group),
	      starter_(GIOPServerStarter::_duplicate(starter)) { }
	virtual ~StarterThread() { }
	virtual void run();
    };
    friend class StarterThread;
    JTCThreadHandle starterThread_;

public:
    
    GIOPServerStarter(ORBInstance_ptr, OCI::Acceptor_ptr, OAInterface_ptr);
    virtual ~GIOPServerStarter();

    static inline GIOPServerStarter_ptr _duplicate(GIOPServerStarter_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline GIOPServerStarter_ptr _nil()
    { return 0; }

    //
    // Remove the worker. This operation is called by GIOPServerWorker
    // and derived classes only.
    //
    void removeWorker(GIOPServerWorker_ptr);

    //
    // Change the state of the worker
    //
    bool setState(State);

    //
    // Run method for starter thread
    //
    void starterRun();
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::GIOPServerStarter_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::GIOPServerStarter_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
