// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/IIOPConfig.h>
#include <OBE/Properties.h>
#include <OBE/Logger.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// IIOPConfig private and protected member implementations
// ----------------------------------------------------------------------

OCI::Param*
IIOPConfig::findParam(const char* name)
{
    for(ULong i = 0 ; i < params_.length() ; i++)
    {
        if(strcmp(params_[i].name, name) == 0)
            return &params_[i];
    }

    ULong len = params_.length() + 1;
    params_.length(len);
    params_[len - 1].name = string_dup(name);
    return &params_[len - 1];
}

void
IIOPConfig::setParam(const char* name, const char* prop)
{
    const char* value = props_ -> getProperty(prop);
    if(value != 0)
    {
        OCI::Param* param = findParam(name);
        param -> value = value;
    }
}

// ----------------------------------------------------------------------
// IIOPConfig constructor
// ----------------------------------------------------------------------

IIOPConfig::IIOPConfig(Properties_ptr props, Logger_ptr logger)
   : props_(Properties::_duplicate(props)),
     logger_(Logger::_duplicate(logger))
{
}

// ----------------------------------------------------------------------
// IIOPConfig public member implementation
// ----------------------------------------------------------------------

//
// Get the parameters for all properties that have been parsed
//
OCI::ParamSeq*
IIOPConfig::getParams()
{
    return new OCI::ParamSeq(params_);
}

//
// Parse the following property:
//
// ooc.iiop.port
//
// This property generally only applies to the RootPOAManager.
//
void
IIOPConfig::parseDefaultPort()
{
    setParam("port", "ooc.iiop.port");
}

//
// Parse the following property:
//
// ooc.iiop.bind
//
// This property generally only applies to the RootPOAManager.
//
void
IIOPConfig::parseDefaultBind()
{
    setParam("bind", "ooc.iiop.bind");
}

//
// Parse the following properties:
//
// ooc.iiop.backlog
// ooc.iiop.host
// ooc.iiop.multi_profile
// ooc.iiop.numeric
//
void
IIOPConfig::parseDefaults()
{
    setParam("host", "ooc.iiop.host");
    setParam("numeric", "ooc.iiop.numeric");
    setParam("multi_profile", "ooc.iiop.multi_profile");
    setParam("backlog", "ooc.iiop.backlog");
}

//
// Parse all properties in the standard order of precedence
//
void
IIOPConfig::parseAll()
{
    parseDefaults();
    parseDefaultPort();
    parseDefaultBind();
}
