// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/Basic.h>
#include <OBE/ORBInstance.h>
#include <OBE/InitialServiceManager.h>
#include <OBE/Client.h>
#include <OBE/ORBControl.h>
#include <OBE/Stream.h>
#include <OBE/RefCounted.h>
#ifndef HAVE_OBE_NO_TRACE
#  include <OBE/CoreTraceLevels.h>
#endif // !HAVE_OBE_NO_TRACE
#include <OBE/Logger.h>  
#include <OBE/Properties.h>
#include <OBE/OBORB.h>

#include <ORBPolicyManager_impl.h>
#include <OBPolicies_impl.h>
#include <ORBMemberFactory.h>
#include <OBPolicies_impl.h>

using namespace CORBA;
using namespace OB;

//
// The ORB implementation map
//
CORBA::ORB::ORBImplMap CORBA::ORB::orbImplMap_(7);

JTCRecursiveMutex CORBA::ORB::orbImplMapMutex_;

const char* CORBA::ORB::ids_[] =
{
    "IDL:omg.org/CORBA/ORB:2.3",
    0
};

const char* OBCORBA::ORB::ids_[] =
{
    "IDL:ooc.com/OBCORBA/ORB:1.0",
    "IDL:omg.org/CORBA/ORB:2.3",
    0
};

// ----------------------------------------------------------------------
// External, non-inline functions
// ----------------------------------------------------------------------

void
OBDuplicate(CORBA::ORB_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(CORBA::ORB_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

void
OBMarshal(CORBA::ORB_ptr, OB::OutputStreamImpl*)
{
    throw CORBA::MARSHAL();
}

void
OBUnmarshal(CORBA::ORB_ptr&, OB::InputStreamImpl*)
{
    throw CORBA::MARSHAL();
}

void
OBDuplicate(OBCORBA::ORB_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(OBCORBA::ORB_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

void
OBMarshal(OBCORBA::ORB_ptr, OB::OutputStreamImpl*)
{
    throw CORBA::MARSHAL();
}

void
OBUnmarshal(OBCORBA::ORB_ptr&, OB::InputStreamImpl*)
{
    throw CORBA::MARSHAL();
}

// ---------------------------------------------------------------------
// IDL:omg.org/CORBA/ORB/InvalidName:1.0
// ---------------------------------------------------------------------

const char*
CORBA::ORB::InvalidName::_rep_id() const
{
    return "IDL:omg.org/CORBA/ORB/InvalidName:1.0";
}

CORBA::Exception*
CORBA::ORB::InvalidName::_OB_clone() const
{
    return new InvalidName(*this);
}

// ----------------------------------------------------------------------
// CORBA::ORB private member functions
// ----------------------------------------------------------------------

void
CORBA::ORB::initializeDefaultPolicies()
{
    //
    // Add the default policies
    //
    ConnectionReusePolicy_impl* reusePolicy =
        new ConnectionReusePolicy_impl(true);

    CORBA::ULong policiesLength = policies_.length();
    policies_.length(policiesLength + 1);
    policies_[policiesLength] = reusePolicy;
    
    RetryPolicy_impl* retryPolicy = new RetryPolicy_impl(OB::RETRY_STRICT);

    policiesLength = policies_.length();
    policies_.length(policiesLength + 1);
    policies_[policiesLength] = retryPolicy;

    //
    // Create the ORBPolicyManager
    //
    PolicyManager_var pm = new ORBPolicyManager_impl(policies_);
    InitialServiceManager_var initServiceManager =
        orbInstance_ -> getInitialServiceManager();
    initServiceManager -> addInitialReference("ORBPolicyManager", pm);
    ObjectFactory_var objectFactory = orbInstance_ -> getObjectFactory();
    objectFactory -> setPolicyManager(pm);
}

// ----------------------------------------------------------------------
// CORBA::ORB constructor and destructor
// ----------------------------------------------------------------------

CORBA::ORB::ORB(int ac, char** av, const char* id,
#ifndef HAVE_OBE_NO_TRACE
	        CoreTraceLevels_ptr coreTraceLevels,
#endif // !HAVE_OBE_NO_TRACE
	        Properties_ptr properties, Logger_ptr logger)
    : id_(id), destroy_(false)
{
    try
    {
	//
	// Create the ORBInstance object
	//
	orbInstance_ = OB::ORBMemberFactory::createORBInstance();
	//
	// Create the ORBControl
	//
	orbControl_ = OB::ORBMemberFactory::createORBControl(orbInstance_);

	//
	// Initialize the ORBInstance object
	//
	orbInstance_ -> initialize(properties, logger
#ifndef HAVE_OBE_NO_TRACE
				   , coreTraceLevels
#endif // !HAVE_OBE_NO_TRACE
				   );

	//
	// Initialize default policies
	//
	initializeDefaultPolicies();

	//
	// Add this ORB to the implementation map -- this must be done
	// last.
	//
	JTCSynchronized sync(orbImplMapMutex_);
	orbImplMap_.put(id_, ORB::_duplicate(this));
    }
    catch(...)
    {
	//
	// Here the same thing as ORB::destroy must be done since
	// although the ORB itself isn't fully initialized all of the
	// ORB components may be.
	//
	if(!CORBA::is_nil(orbControl_))
	{
	    orbControl_ -> shutdownServerClient();
	    orbControl_ -> destroy();
	}
	if(!CORBA::is_nil(orbInstance_))
	{
	    orbInstance_ -> destroy();
	    orbInstance_ = ORBInstance::_nil();
	}
	_OB_setRef(0);
	throw;
    }
}

CORBA::ORB::~ORB()
{
    //
    // Check whether the ORB has been properly destroyed
    //
    if(!destroy_)
    {
#ifndef HAVE_OBE_NO_TRACE
	Logger_var logger = orbInstance_ -> getLogger();
	logger -> warning("ORB::destroy() was not called. "
			  "This may result in memory leaks.");
#endif // !HAVE_OBE_NO_TRACE
    }
}

// ---------------------------------------------------------------------
// CORBA::ORB public members
// ---------------------------------------------------------------------

CORBA::ORB_ptr
CORBA::ORB::_narrow(CORBA::Object_ptr p)
{
    if(!CORBA::is_nil(p))
    {
        ORB_ptr v = dynamic_cast<ORB_ptr>(p);
        if(v)
            return _duplicate(v);
    }

    return _nil();
}

const char**
CORBA::ORB::_OB_staticIds()
{
    return ids_;
}

char*
CORBA::ORB::object_to_string(Object_ptr p)
{
    {
	JTCSynchronized synchronized(*this);
	if(destroy_)
	    throw OBJECT_NOT_EXIST();
    }

    RefCountIOR_var ior;
    
    if(is_nil(p))
    {
	IOP::IOR empty;
	ior = new RefCountIOR(empty);
    }
    else
    {
        if(p -> _OB_isLC())
            throw BAD_PARAM();
	
	ior = p -> _OB_origIOR(); 
    }

    OutputStreamImpl out;
    out._OB_writeEndian();
    ior -> value._OB_marshal(&out);
    String_var s = OctetsToAscii(out._OB_bufferImpl() -> data(),
				 out._OB_bufferImpl() -> length());
    
    static const char* prefix = "IOR:";
    char* result = string_alloc(strlen(prefix) + strlen(s));
    strcpy(result, prefix);
    strcat(result, s);

    return result;
}

CORBA::Object_ptr
CORBA::ORB::string_to_object(const char* ior)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    ObjectFactory_var objectFactory = orbInstance_ -> getObjectFactory();
    return objectFactory -> stringToObject(ior);
}

CORBA::Object_ptr
CORBA::ORB::resolve_initial_references(const char* identifier)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    InitialServiceManager_var initServiceManager =
        orbInstance_ -> getInitialServiceManager();

    Object_var obj;

    try
    {
        obj = initServiceManager -> resolveInitialReferences(identifier);
    }
    catch(const InvalidName&)
    {
        //
        // If the service is the RootPOA and it hasn't yet been
        // initialized, create it. We could put in some automatic
        // method here for late binding of objects at some later
        // point.
        //
        if(strcmp(identifier, "RootPOA") == 0)
        {
	    //
	    // Initialize the root POA
	    //
	    orbControl_ -> initializeRootPOA();

            return resolve_initial_references(identifier);
        }
        else
            throw;
    }

    return obj._retn();
}

void
CORBA::ORB::register_initial_reference(const char* name, Object_ptr obj)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    InitialServiceManager_var initServiceManager =
        orbInstance_ -> getInitialServiceManager();
    initServiceManager -> addInitialReference(name, obj);
}

void
CORBA::ORB::run()
{
    //
    // Ensure that the ORB mutex is not locked during the call to
    // ORBControl methods
    //
    if(destroy_)
	throw OBJECT_NOT_EXIST();
    orbControl_ -> run();
}

//
// ASYNC SAFE if called with wait_for_completion = false
//
void
CORBA::ORB::shutdown(Boolean wait_for_completion)
{
    //
    // Ensure that the ORB mutex is not locked during the call to
    // ORBControl methods
    //
    if(destroy_)
	throw OBJECT_NOT_EXIST();
    orbControl_ -> shutdownServer(wait_for_completion);
}

void
CORBA::ORB::destroy()
{
    //
    // From the specification:
    //
    // This operation destroys the ORB so that its resources can be
    // reclaimed by the application.  Any operation invoked on a
    // destroyed ORB reference will raise the OBJECT_NOT_EXIST
    // exception.  Once an ORB has been destroyed, another call to
    // ORB_init with the same ORBid will return a reference to a newly
    // constructed ORB.
    //
    // If destroy is called on an ORB that has not been shut down, it
    // will start the shut down process and block until the ORB has
    // shut down before it destroys the ORB.  If an application calls
    // destroy in a thread that is currently servicing an invocation,
    // the BAD_INV_ORDER system exception will be raised with the OMG
    // minor code 3, since blocking would result in a deadlock.
    //
    // For maximum portability and to avoid resource leaks, an
    // application should always call shutdown and destroy on all ORB
    // instances before exiting.
    //
    
    JTCSynchronized synchronized(*this);

    //
    // Has the ORB been destroyed yet?
    //
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    //
    // Shutdown both the server & client side of the ORB
    //
    orbControl_ -> shutdownServerClient();

    //
    // Destroy the ORBControl. Don't set to _nil.
    //
    orbControl_ -> destroy();
    //orbControl_ = ORBControl::_nil();
    
    //
    // Destroy the ORBInstance object
    //
    orbInstance_ -> destroy();
    orbInstance_ = ORBInstance::_nil();

    //
    // Mark the ORB as destroyed
    //
    destroy_ = true;

    //
    // Remove the ORB from the implementation map
    //
    JTCSynchronized sync(orbImplMapMutex_);
    orbImplMap_.remove(id_);
}

// ----------------------------------------------------------------------
// OBCORBA::ORB constructor and destructor
// ----------------------------------------------------------------------

OBCORBA::ORB::ORB(int ac, char** av, const char* id,
#ifndef HAVE_OBE_NO_TRACE
	          CoreTraceLevels_ptr coreTraceLevels,
#endif // !HAVE_OBE_NO_TRACE
	          Properties_ptr properties, Logger_ptr logger)
    : CORBA::ORB(ac, av, id,
#ifndef HAVE_OBE_NO_TRACE
	         coreTraceLevels,
#endif // !HAVE_OBE_NO_TRACE
		 properties, logger)
{
}

OBCORBA::ORB::~ORB()
{
}

// ---------------------------------------------------------------------
// OBCORBA::ORB public members
// ---------------------------------------------------------------------

OBCORBA::ORB_ptr
OBCORBA::ORB::_narrow(CORBA::Object_ptr p)
{
    if(!CORBA::is_nil(p))
    {
        ORB_ptr v = dynamic_cast<ORB_ptr>(p);
        if(v)
            return _duplicate(v);
    }

    return _nil();
}

const char**
OBCORBA::ORB::_OB_staticIds()
{
    return ids_;
}

OB::PropertySet
OBCORBA::ORB::properties()
{
    JTCSynchronized synchronized(*this);

    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return orbInstance_ -> getProperties();
}

OB::Logger_ptr
OBCORBA::ORB::logger()
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return orbInstance_ -> getLogger();
}

OB::ConnectionReusePolicy_ptr
OBCORBA::ORB::create_connection_reuse_policy(CORBA::Boolean value)
{
    return new OB::ConnectionReusePolicy_impl(value);
}

OB::ProtocolPolicy_ptr
OBCORBA::ORB::create_protocol_policy(OCI::ProtocolId value)
{
    return new OB::ProtocolPolicy_impl(value);
}

OB::RetryPolicy_ptr
OBCORBA::ORB::create_retry_policy(CORBA::Short value)
{
    return new OB::RetryPolicy_impl(value);
}

OB::TimeoutPolicy_ptr
OBCORBA::ORB::create_timeout_policy(CORBA::ULong value)
{
    return new OB::TimeoutPolicy_impl(value);
}

OB::ConnectTimeoutPolicy_ptr
OBCORBA::ORB::create_connection_timeout_policy(CORBA::ULong value)
{
    return new OB::ConnectTimeoutPolicy_impl(value);
}

OB::RequestTimeoutPolicy_ptr
OBCORBA::ORB::create_request_timeout_policy(CORBA::ULong value)
{
    return new OB::RequestTimeoutPolicy_impl(value);
}

// ----------------------------------------------------------------------
// Get an ORBInstance, given an ORB_ptr
// ----------------------------------------------------------------------

ORBInstance_ptr
OB::GetORBInstance(CORBA::ORB_ptr p)
{
    if(is_nil(p))
	return ORBInstance::_nil();
    
    OBCORBA::ORB* impl = dynamic_cast<OBCORBA::ORB*>(p);
    if(!impl)
	throw INITIALIZE();
    
    return ORBInstance::_duplicate(impl -> orbInstance_);
}
