// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/Basic.h>
#include <OBE/ORBInstance.h>
#include <OBE/InitialServiceManager.h>
#include <OBE/Client.h>
#include <OBE/ORBControl.h>
#include <OBE/Stream.h>
#include <OBE/RefCounted.h>
#ifndef HAVE_OBE_NO_TRACE
#  include <OBE/CoreTraceLevels.h>
#endif // !HAVE_OBE_NO_TRACE
#include <OBE/Logger.h>  
#include <OBE/Properties.h>
#include <OBE/OBORB.h>

#include <ORBPolicyManager_impl.h>
#include <OBPolicies_impl.h>
#include <ORBMemberFactory.h>
#include <OBPolicies_impl.h>

using namespace CORBA;
using namespace OB;

// ---------------------------------------------------------------------
// OBCORBA::ORB public members
// ---------------------------------------------------------------------


Policy_ptr
CORBA::ORB::create_policy(PolicyType type, const CORBA::Any& val)
{
    //
    // A case stmt would be better here, but egcs complains: case
    // label `CONNECTION_REUSE_POLICY_ID' does not reduce to an
    // integer constant
    //
    if(type == CONNECTION_REUSE_POLICY_ID)
    {
	Boolean b;
	if(val >>= Any::to_boolean(b))
        {
	    return new ConnectionReusePolicy_impl(b);
        }
        throw PolicyError(BAD_POLICY_TYPE);
    }
    
    if(type == PROTOCOL_POLICY_ID)
    {
        OCI::ProtocolId id;
        if(val >>= id)
        {
            return new ProtocolPolicy_impl(id);
        }
        throw PolicyError(BAD_POLICY_TYPE);
    }
    
    if(type == RETRY_POLICY_ID)
    {
        Short v;
        if(val >>= v)
        {
            if(v > RETRY_ALWAYS)
                throw PolicyError(BAD_POLICY_VALUE);
            return new RetryPolicy_impl(v);
        }
        throw PolicyError(BAD_POLICY_TYPE);
    }

    if(type == TIMEOUT_POLICY_ID)
    {
        ULong t;
        if(val >>= t)
        {
            return new TimeoutPolicy_impl(t);
        }
        throw PolicyError(BAD_POLICY_TYPE);
    }

    if(type == CONNECT_TIMEOUT_POLICY_ID)
    {
        ULong t;
        if(val >>= t)
        {
            return new ConnectTimeoutPolicy_impl(t);
        }
        throw PolicyError(BAD_POLICY_TYPE);
    }

    if(type == REQUEST_TIMEOUT_POLICY_ID)
    {
        ULong t;
        if(val >>= t)
        {
            return new RequestTimeoutPolicy_impl(t);
        }
        throw PolicyError(BAD_POLICY_TYPE);
    }
    
    throw PolicyError(BAD_POLICY);
    return Policy::_nil(); // Some compilers need this
}
