// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OCICLIENT_IIOP_IMPL_H
#define OB_OCICLIENT_IIOP_IMPL_H

#include <OBE/Object.h>
#include <OBE/Except.h>
#include <OBE/OCIClient_IIOP.h>
#include <OBE/Net.h>

namespace OCI
{

namespace IIOP
{

//
// The Transport_impl class
//
class Transport_impl : public Transport
{
    //
    // Hide copy-constructor and assignment operator
    //
    Transport_impl(const Transport_impl&);
    void operator=(const Transport_impl&);

protected:

    Handle fd_; // The handle
    CORBA::Boolean block_; // Set to true if blocking IO is selected
    
    void setBlock(CORBA::Boolean); // Set blocking/non-blocking behavior

public:

    Transport_impl(Handle);
    virtual ~Transport_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException)
    {
        return TAG_IIOP;
    }

    virtual ProfileId tag()
        throw(CORBA::SystemException)
    {
        return IOP::TAG_INTERNET_IOP;
    }
    
    virtual Handle handle()
        throw(CORBA::SystemException)
    {
        return fd_;
    }
    
    virtual void close()
	throw(CORBA::SystemException);
    
    virtual void shutdown()
	throw(CORBA::SystemException);
    
    virtual void receive(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);
    
    virtual CORBA::Boolean receive_detect(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);
    
    virtual void receive_timeout(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);
    
    virtual void send(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);
    
    virtual CORBA::Boolean send_detect(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);
    
    virtual void send_timeout(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);
};

//
// The Connector_impl class
//
class Connector_impl : public Connector
{
    //
    // Hide copy-constructor and assignment operator
    //
    Connector_impl(const Connector_impl&);
    void operator=(const Connector_impl&);

protected:

    CORBA::String_var host_; // The host
    CORBA::UShort port_; // The port
    Handle fd_; // The handle

    void close(); // Close the connector

public:

    Connector_impl(const char*, CORBA::UShort);
    virtual ~Connector_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException)
    {
        return TAG_IIOP;
    }

    virtual ProfileId tag()
        throw(CORBA::SystemException)
    {
        return IOP::TAG_INTERNET_IOP;
    }
    
    virtual Handle handle()
        throw(CORBA::SystemException)
    {
        return fd_;
    }
    
    virtual OCI::Transport_ptr connect()
        throw(CORBA::SystemException);

    virtual OCI::Transport_ptr connect_timeout(CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::ProfileInfoSeq* get_usable_profiles(const IOR&,
						     const CORBA::PolicyList&)
        throw(CORBA::SystemException);
    
    virtual CORBA::Boolean equal(Connector_ptr)
        throw(CORBA::SystemException);
};

//
// The ConFactory_impl class
//
class ConFactory_impl : public ConFactory
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConFactory_impl(const ConFactory_impl&);
    void operator=(const ConFactory_impl&);

public:

    ConFactory_impl();
    virtual ~ConFactory_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException)
    {
        return TAG_IIOP;
    }

    virtual ProfileId tag()
        throw(CORBA::SystemException)
    {
        return IOP::TAG_INTERNET_IOP;
    }
    
    virtual OCI::ConnectorSeq* create_connectors(const IOR&,
						 const CORBA::PolicyList&)
        throw(CORBA::SystemException);
    
    virtual CORBA::Boolean equivalent(const IOR&, const IOR&)
        throw(CORBA::SystemException);
    
    virtual CORBA::ULong hash(const IOR&, CORBA::ULong)
        throw(CORBA::SystemException);
};

} // End of namespace IIOP

} // End of namespace OCI

#endif
