// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OCIClient_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ConFactoryRegistry_impl public member implementation
// ----------------------------------------------------------------------

void
OCI::ConFactoryRegistry_impl::add_factory(ConFactory_ptr factory)
    throw(OCI::FactoryAlreadyExists,
          SystemException)
{
    JTCSynchronized synchronized(*this);

    ProfileId id = factory -> id();

    for(CORBA::ULong i = 0 ; i < factoryCount_ ; ++i)
    {
        if(factories_[i] -> id() == id)
	    throw FactoryAlreadyExists(id);
    }

    if(factoryCount_ >= MAX_CON_FACTORY)
        throw CORBA::IMP_LIMIT();

    factories_[factoryCount_++] = ConFactory::_duplicate(factory);
}

OCI::ConFactory_ptr
OCI::ConFactoryRegistry_impl::get_factory(ProfileId id)
    throw(OCI::NoSuchFactory,
          SystemException)
{
    JTCSynchronized synchronized(*this);

    for(CORBA::ULong i = 0 ; i < factoryCount_ ; ++i)
    {
        if(factories_[i] -> id() == id)
	    return ConFactory::_duplicate(factories_[i]);
    }
    
    throw NoSuchFactory(id);
}

OCI::ConFactorySeq*
OCI::ConFactoryRegistry_impl::get_factories()
    throw(SystemException)
{
    OCI::ConFactorySeq* seq = new ConFactorySeq(factoryCount_);
    seq -> length(factoryCount_);

    for(CORBA::ULong i = 0 ; i < factoryCount_ ; ++i)
        (*seq)[i] = ConFactory::_duplicate(factories_[i]);

    return seq;
}
