// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OCICLIENT_IMPL_H
#define OB_OCICLIENT_IMPL_H

#include <OBE/Object.h>
#include <OBE/Except.h>
#include <OBE/OCIClient.h>
#include <OBE/Defs.h>

namespace OB
{

class ORBInstance; // For the friend declaration

} // End of namespace OB

namespace OCI
{

//
// The ConFactoryRegistry_impl class
//
class ConFactoryRegistry_impl : virtual public ConFactoryRegistry,
				public JTCMutex
{
    //
    // All connector factories
    //
    ConFactory_var factories_[MAX_CON_FACTORY];
    CORBA::ULong factoryCount_;

    //
    // Hide copy-constructor and assignment operator
    //
    ConFactoryRegistry_impl(const ConFactoryRegistry_impl&);
    void operator=(const ConFactoryRegistry_impl&);

    ConFactoryRegistry_impl() : factoryCount_(0) { }
    virtual ~ConFactoryRegistry_impl() { }

    // ORBInstance creates ConFactoryRegistry_impl
    friend class ::OB::ORBInstance;

public:

    //
    // Standard IDL to C++ Mapping
    //
    virtual void add_factory(ConFactory_ptr)
        throw(FactoryAlreadyExists,
              CORBA::SystemException);

    virtual ConFactory_ptr get_factory(OCI::ProtocolId)
        throw(NoSuchFactory,
              CORBA::SystemException);

    virtual ConFactorySeq* get_factories()
        throw(CORBA::SystemException);
};

} // End of namespace OCI

#endif
