// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OCI_IIOP_IMPL_H
#define OB_OCI_IIOP_IMPL_H

#include <OBE/Object.h>
#include <OBE/Except.h>
#include <OBE/OCI_IIOP.h>
#include <OBE/Net.h>

#include <OCIClient_IIOP_impl.h>

namespace OCI
{

namespace IIOP
{

//
// The Acceptor_impl class
//
class Acceptor_impl : public Acceptor
{
    //
    // Hide copy-constructor and assignment operator
    //
    Acceptor_impl(const Acceptor_impl&);
    void operator=(const Acceptor_impl&);

protected:

    CORBA::StringSeq hosts_; // The hosts
    bool multiProfile_; // Use multiple profiles?
    CORBA::UShort port_; // The port
    CORBA::UShort backlog_; // The listen backlog
    Handle fd_; // The handle
    struct sockaddr_in localAddress_; // The local address

    //
    // The corresponding info class is a friend
    //
    friend class AcceptorInfo_impl;

public:

    Acceptor_impl(const CORBA::StringSeq&, bool, CORBA::UShort, CORBA::UShort);
    Acceptor_impl(const char*, const CORBA::StringSeq&, bool, CORBA::UShort,
                  CORBA::UShort);
    virtual ~Acceptor_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException)
    {
        return TAG_IIOP;
    }

    virtual ProfileId tag()
        throw(CORBA::SystemException)
    {
        return IOP::TAG_INTERNET_IOP;
    }
    
    virtual Handle handle()
        throw(CORBA::SystemException)
    {
        return fd_;
    }
    
    virtual void close()
	throw(CORBA::SystemException);
    
    virtual void shutdown()
	throw(CORBA::SystemException);
    
    virtual void listen()
        throw(CORBA::SystemException);
    
    virtual OCI::Transport_ptr accept(CORBA::Boolean)
        throw(CORBA::SystemException);
    
    virtual OCI::Transport_ptr connect_self()
        throw(CORBA::SystemException);
    
    virtual void add_profiles(const ProfileInfo&, IOR&)
        throw(CORBA::SystemException);

    virtual ProfileInfoSeq* get_local_profiles(const IOR&)
        throw(CORBA::SystemException);
};

//
// The AccFactory_impl class
//
class AccFactory_impl : public AccFactory
{
    //
    // Hide copy-constructor and assignment operator
    //
    AccFactory_impl(const AccFactory_impl&);
    void operator=(const AccFactory_impl&);

public:

    AccFactory_impl();
    virtual ~AccFactory_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual OCI::ProtocolId id()
        throw(CORBA::SystemException)
    {
        return TAG_IIOP;
    }

    virtual ProfileId tag()
        throw(CORBA::SystemException)
    {
        return IOP::TAG_INTERNET_IOP;
    }

    virtual OCI::Acceptor_ptr create_acceptor(const OCI::ParamSeq&)
        throw(OCI::InvalidParam, CORBA::SystemException);
};

} // End of namespace IIOP

} // End of namespace OCI

#endif
