// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OCI_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// AccFactoryRegistry_impl public member implementation
// ----------------------------------------------------------------------

void
OCI::AccFactoryRegistry_impl::add_factory(AccFactory_ptr factory)
    throw(OCI::FactoryAlreadyExists,
          SystemException)
{
    JTCSynchronized synchronized(*this);

    ProfileId id = factory -> id();

    for(CORBA::ULong i = 0 ; i < factoryCount_ ; ++i)
    {
        if(factories_[i] -> id() == id)
	    throw FactoryAlreadyExists(id);
    }

    if(factoryCount_ >= MAX_ACC_FACTORY)
        throw CORBA::IMP_LIMIT();

    factories_[factoryCount_++] = AccFactory::_duplicate(factory);
}

OCI::AccFactory_ptr
OCI::AccFactoryRegistry_impl::get_factory(ProfileId id)
    throw(OCI::NoSuchFactory,
          SystemException)
{
    JTCSynchronized synchronized(*this);

    for(CORBA::ULong i = 0 ; i < factoryCount_ ; ++i)
    {
        if(factories_[i] -> id() == id)
	    return AccFactory::_duplicate(factories_[i]);
    }

    throw NoSuchFactory(id);
}

OCI::AccFactorySeq*
OCI::AccFactoryRegistry_impl::get_factories()
    throw(SystemException)
{
    OCI::AccFactorySeq* seq = new AccFactorySeq(factoryCount_);
    seq -> length(factoryCount_);

    for(CORBA::ULong i = 0 ; i < factoryCount_ ; ++i)
        (*seq)[i] = AccFactory::_duplicate(factories_[i]);

    return seq;
}
