// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/ORBControl.h>
#include <OBE/ORBInstance.h>

#include <ORBControlC.h>

using namespace OB;
using namespace CORBA;

// ----------------------------------------------------------------------
// ORBControlC private and protected member implementation
// ----------------------------------------------------------------------

void
ORBControlC::completeServerShutdown()
    throw()
{
    assert(false);
}

void
ORBControlC::validateState()
    throw(SystemException)
{
    assert(false);
}

void
ORBControlC::blockServerShutdownComplete()
    throw()
{
    assert(false);
}

bool
ORBControlC::insideMethodInvocation()
    throw()
{
    return false;
}

// ----------------------------------------------------------------------
// ORBControl constructor and destructor
// ----------------------------------------------------------------------

ORBControlC::ORBControlC(ORBInstance_ptr instance)
    : ORBControl(instance)
{
}

ORBControlC::~ORBControlC()
{
}

// ----------------------------------------------------------------------
// ORBControlC public member implementation
// ----------------------------------------------------------------------

void
ORBControlC::initializeRootPOA()
{
    assert(false);
}

void
ORBControlC::run()
    throw(SystemException)
{
    assert(false);
}

void
ORBControlC::shutdownServer(bool waitForCompletion)
    throw(SystemException)
{
    state_ = StateServerShutdown;
}

void
ORBControlC::shutdownServerClient()
    throw(SystemException)
{
    JTCSynchronized synchronized(*this);

    //
    // The ORB destroys this object, so it's an initialization error
    // if the this operation is called after ORB destruction
    //
    if(state_ == StateDestroyed)
        throw INITIALIZE(MinorORBDestroyed, COMPLETED_NO);

    //
    // If the ORB client side is already shutdown, then we're done
    //
    if(state_ == StateClientShutdown)
        return;

    //
    // Shutdown the client side. Continue to dispatch events until all
    // client type event handlers have unregistered.
    //
    ClientManager_var clientManager = orbInstance_ -> getClientManager();
    if(!is_nil(clientManager))
        clientManager -> destroy();

    //
    // Mark the ORB is client side as shutdown
    //
    state_ = StateClientShutdown;
}
