// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/ORBInstance.h>
#include <OBE/InitialServiceManager.h>
#include <OBE/Properties.h>
#include <OBE/Client.h>
#include <OBE/OCI_IIOP_init.h>
#include <OBE/OBPOAManager.h>
#include <OBE/BootManager.h>

#include <ORBInstanceS.h>
#include <OCI_impl.h>
#include <ClientS.h>

using namespace CORBA;
using namespace OB;

JTCInitialize* ORBInstanceS::globalStateJTC_ = 0;

// ----------------------------------------------------------------------
// ORBInstanceS private member implementation
// ----------------------------------------------------------------------

void
ORBInstanceS::initializeHook()
{
    //
    // Create the AccFactoryRegistry
    //
    accFactoryRegistry_ = new OCI::AccFactoryRegistry_impl();

    //
    // Create the server worker group
    //
    serverWorkerGroup_ = new JTCThreadGroup("ServerWorkers");

    //
    // Add initial services to list
    //
    InitialServiceManager_var initServiceManager = getInitialServiceManager();
    initServiceManager -> addInitialReference("OCIAccFactoryRegistry",
					      accFactoryRegistry_);

    //
    // Initialize the IIOP OCI plug-in, if necessary
    //
    Properties_var properties = getProperties();
    const char* initIIOP = properties -> getProperty("ooc.orb.init_iiop");
    if(initIIOP == 0 ||
       strcmp(initIIOP, "both") == 0 ||
       strcmp(initIIOP, "server") == 0)
    {
	OCI::IIOP::initServer(accFactoryRegistry_);
    }

    //
    // Create the POAManager
    //
    poaManager_ = new OBPortableServer::POAManager(this);

    //
    // Create Boot Manager
    //
    bootManager_ = new BootManager;
    initServiceManager -> addInitialReference("BootManager", bootManager_);
}

void
ORBInstanceS::destroyHook()
{
    //
    // Destroy the Boot Manager
    //
    if(!CORBA::is_nil(bootManager_))
        bootManager_ -> _OB_destroy();
    bootManager_ = OB::BootManager::_nil();

    //
    // Destroy the POAManager
    //
    if(!CORBA::is_nil(poaManager_))
	poaManager_ -> _OB_destroy();
    poaManager_ = OBPortableServer::POAManager::_nil();

    //
    // Destroy the server worker group
    //
    if(serverWorkerGroup_)
    {
	try
	{
	    serverWorkerGroup_ -> destroy();
	}
	catch(const JTCIllegalThreadStateException&)
	{
	    assert(false);
	}
    }

    //
    // Destroy the AccFactoryRegistry
    //
    //accFactoryRegistry_ -> destroy(); // No destroy operation defined
    accFactoryRegistry_ = OCI::AccFactoryRegistry::_nil();
}

ClientManager_ptr
ORBInstanceS::createClientManager(ORBInstance_ptr)
{
    return new ClientManagerS(this);
}

// ----------------------------------------------------------------------
// ORBInstanceS constructor and destructor
// ----------------------------------------------------------------------

ORBInstanceS::ORBInstanceS()
    : serverWorkerGroup_(0)
{
    if(globalStateMutex_ != 0)
	globalStateMutex_ -> lock();

    //
    // Initialize JTC, if necessary
    //
    if(!JTCInitialize::initialized())
        globalStateJTC_ = new JTCInitialize();

    if(globalStateMutex_ != 0)
	globalStateMutex_ -> unlock();
}

ORBInstanceS::~ORBInstanceS()
{
    if(globalStateMutex_ != 0)
	globalStateMutex_ -> lock();

    if(globalStateCounter_ <= 1)
        delete globalStateJTC_;

    if(globalStateMutex_ != 0)
	globalStateMutex_ -> unlock();
}

// ----------------------------------------------------------------------
// ORBInstanceS public member implementation
// ----------------------------------------------------------------------

OBPortableServer::POAManager_ptr
ORBInstanceS::getPOAManager()
{
    return OBPortableServer::POAManager::_duplicate(poaManager_);
}

JTCThreadGroupHandle
ORBInstanceS::getServerWorkerGroup()
{
    return serverWorkerGroup_;
}

OCI::AccFactoryRegistry_ptr
ORBInstanceS::getAccFactoryRegistry()
{
    return OCI::AccFactoryRegistry::_duplicate(accFactoryRegistry_);
}

BootManager_ptr
ORBInstanceS::getBootManager()
{
    return BootManager::_duplicate(bootManager_);
}
