// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef ORB_INSTANCE_S_H
#define ORB_INSTANCE_S_H

#include <OBE/ORBInstance_fwd.h>
#include <OBE/OBPOAManager_fwd.h>
#include <OBE/OCI_fwd.h>
#include <OBE/BootManager_fwd.h>

namespace OB
{
    
//
// The ORBInstanceS class. This class implements the ORBInstance
// abstract class and provides items needed by server applications.
//
class ORBInstanceS : public ORBInstance
{
    //
    // Hide copy-constructor and assignment operator
    //
    ORBInstanceS(const ORBInstanceS&);
    void operator=(const ORBInstanceS&);

    //
    // ORBInstance members
    //
    // NOTE: For each new member "X_var x_", add the following to
    // the destroy() operation:
    //
    // x_ -> destroy(); // Only if X has an operation destroy()
    // x_ = X::_nil(); // To resolve potential cyclic dependencies
    //
    OBPortableServer::POAManager_var poaManager_;
    OCI::AccFactoryRegistry_var accFactoryRegistry_;
    BootManager_var bootManager_;
    JTCThreadGroupHandle serverWorkerGroup_;

    //
    // Global state management
    //
    static JTCInitialize* globalStateJTC_;

    //
    // Initialize and destroy hooks used by derived classes
    // (called by initialize() and destroy() respectively)
    //
    virtual void initializeHook(); 
    virtual void destroyHook();

    //
    // Create the ClientManager
    //
    virtual ClientManager_ptr createClientManager(ORBInstance_ptr);

public:

    ORBInstanceS();
    ~ORBInstanceS();

    //
    // Get the POAManager
    //
    OBPortableServer::POAManager_ptr getPOAManager();

    //
    // Get the OCI acceptor factory registry
    //
    OCI::AccFactoryRegistry_ptr getAccFactoryRegistry();

    //
    // Get the BootManager
    //
    BootManager_ptr getBootManager();

    //
    // Get the server worker thread group
    //
    JTCThreadGroupHandle getServerWorkerGroup();
};

} // End of namespace OB

#endif
