// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/ObjectKey.h>

#include <stdlib.h>

using namespace OB;
using namespace CORBA;

// ----------------------------------------------------------------------
// ObjectKey OB namespace functions
// ----------------------------------------------------------------------

//
// OBE object key: 0 + iter(4) + slot(4)
//

OCI::ObjectKey*
OB::CreateObjectKey(const ObjectKeyData& id)
{
    OCI::ObjectKey_var key = new OCI::ObjectKey(9);
    key -> length(9);

    char* data = (char*)key -> get_buffer();

    *data++ = '\0';

    memcpy(data, &id.iter, sizeof(id.iter));
    data += 4;
    
    memcpy(data, &id.slot, sizeof(id.slot));
    
    return key._retn();
}

//
// Parse an object key, filling in the fields of the ObjectKeyData.
// Return false if the object-key cannot be parsed, true otherwise.
//
bool
OB::ParseObjectKey(const OCI::ObjectKey& key,
                   ObjectKeyData& keyData)
{
    const Octet* data = key.get_buffer();

    if(key.length() != 9 || *data++ != '\0')
        return false;
    
    //
    // It's not directly possible to do this since the buffer isn't
    // correctly aligned under DEC or SGI.
    //
    //keyData.iter = *(ULong*)data;

    keyData.iter = 0;
    memcpy(&keyData.iter, data, 4);
    data += 4;

    keyData.slot = 0;
    memcpy(&keyData.slot, data, 4);

    return true;
}

// ----------------------------------------------------------------------
// ObjectKeyData constructor
// ----------------------------------------------------------------------

ObjectKeyData::ObjectKeyData()
{
}

ObjectKeyData::ObjectKeyData(ULong _iter, ULong _slot)
    : iter(_iter),
      slot(_slot)
{
}
