// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/ORBInstance.h>
#include <OBE/RefCounted.h>
#include <OBE/NativeTypes.h>
#include <OBE/Except.h>
#include <OBE/Object.h>
#include <OBE/OCI.h>
#include <OBE/Upcall.h>
#include <OBE/Stream.h>
#include <OBE/POA.h>
#include <OBE/OBPOAManager.h>
#include <OBE/BootManager.h>
#include <OBE/ObjectKey.h>

#include <POAOAInterface_impl.h>
#include <ORBInstanceS.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OAInterface_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(OAInterface_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// POAOAInterface_impl constructor
// ----------------------------------------------------------------------

POAOAInterface_impl::POAOAInterface_impl(
    PortableServer::POAManager_ptr poaManager,
    ORBInstance_ptr orbInstance)
    : orbInstance_(ORBInstance::_duplicate(orbInstance)),
      poaManager_(poaManager)
{
}

bool
POAOAInterface_impl::setupUpcall(UpcallImpl* upcall)
    throw()
{
    ObjectKeyData data;
    if(ParseObjectKey(upcall -> key(), data))
    {
	PortableServer::POA_ptr poa = poaManager_ -> _OB_POA_nodup();
	if(!is_nil(poa))
	{
	    upcall -> setDispatchInfo(poa, data);
	    return true;
	}
    }
    else
    {
	//
	// Check to see if the BootManager knows of a reference for
	// the ObjectKey. If so, forward the request.
	//
	if(is_nil(bootManager_))
	{
	    ORBInstanceS* orbInstance = 
		dynamic_cast<ORBInstanceS*>(orbInstance_.in());
	    bootManager_ = orbInstance -> getBootManager();
	}
	RefCountIOR_var ior = bootManager_ -> _OB_locate(upcall -> key());
	if(!is_nil(ior))
	{
	    upcall -> setLocationForward(ior);
	    return false;
	}
    }

    try
    {
	if(strcmp(upcall -> operation(), "_non_existent") == 0 ||
	   strcmp(upcall -> operation(), "_not_existent") == 0)
	{
	    upcall -> preUnmarshal();
	    upcall -> postUnmarshal();
	    OutputStreamImpl* out = upcall -> preMarshal();
	    out -> write_boolean(true);
	    upcall -> postMarshal();
	}
	else
	    throw OBJECT_NOT_EXIST();
    }
    catch(const SystemException& ex)
    {
	upcall -> setSystemException(ex);
    }

    return false;
}

OAInterface::LocateStatus
POAOAInterface_impl::findByKey(const OCI::ObjectKey& key, RefCountIOR_out ior)
    throw()
{
    ObjectKeyData data;
    if(ParseObjectKey(key, data))
    {
	try
	{
	    PortableServer::POA_ptr poa = poaManager_ -> _OB_POA_nodup();
	    if(!is_nil(poa))
	    {
		//PortableServer::POA_impl* poaImpl =
		//  dynamic_cast<PortableServer::POA_impl*>(poa.in());
		return poa -> _OB_locateServant(data)
		? OAInterface::OBJECT_HERE
		: OAInterface::UNKNOWN_OBJECT;
	    }
	}
	catch(const TRANSIENT&)
	{
	}
	catch(const LocationForward& fwd)
	{
	    ior = RefCountIOR::_duplicate(fwd.ior);
	    return OAInterface::OBJECT_FORWARD;
	}
    }
    return OAInterface::UNKNOWN_OBJECT;
}

OCI::ProfileInfoSeq*
POAOAInterface_impl::getUsableProfiles(const OCI::IOR& ior, const PolicyList&)
    throw()
{
    OCI::AcceptorSeq_var acceptors = poaManager_ -> _OB_getAcceptors();

    OCI::ProfileInfoSeq_var seq = new OCI::ProfileInfoSeq;
    for(ULong i = 0 ; i < acceptors -> length() ; i++)
    {
        OCI::ProfileInfoSeq_var seq2 = acceptors[i] -> get_local_profiles(ior);
        for(ULong j = 0 ; j < seq2 -> length() ; j++)
        {
            seq -> length(seq -> length() + 1);
            seq[seq -> length() - 1] = seq2[j];
        }
    }

    return seq._retn();
}

void
POAOAInterface_impl::activate()
    throw()
{
}
