// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/NativeTypes.h>
#include <OBE/MarshalStubImpl.h>
#include <OBE/OBORB.h>
#include <OBE/ORB_init.h>
#include <OBE/POAServantBase.h>
#include <OBE/ORBInstance.h>
#include <OBE/InitialServiceManager.h>
#include <OBE/Upcall.h>
#include <OBE/Stream.h>
#include <OBE/POA.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBRelease(PortableServer::ServantBase* servant)
{
    if(servant)
        servant -> _remove_ref();
}

void
OBDuplicate(PortableServer::ServantBase* servant)
{
    if(servant)
        servant -> _add_ref();
}

// ----------------------------------------------------------------------
// ServantBase private and protected member implementation
// ----------------------------------------------------------------------

//
// TODO: Do we need PIUpcall, etc versions of this?
//
void
PortableServer::ServantBase::_OB_dispatchBase(UpcallImpl* _ob_up)
{
    static const char* names[] =
    {
	"_interface",
        "_is_a",
	"_non_existent"
    };
    static const ULong num = 3;

    switch(_OB_findOperation(_ob_up, names, num))
    {

    case 0: // _interface
    {
        _ob_up -> preUnmarshal();
        _ob_up -> postUnmarshal();
        _ob_up -> preMarshal();
	_ob_up -> marshalEx(NO_IMPLEMENT());
        _ob_up -> postMarshal();
        return;
    }
    case 1: // _is_a
    {
	const char* _ob_a0;
	OB::InputStreamImpl* _ob_in = _ob_up -> preUnmarshal();
	try
	{
	    _ob_a0 = _ob_in -> read_string_nodup();
	}
	catch(const CORBA::SystemException& _ob_ex)
	{
	    _ob_up -> unmarshalEx(_ob_ex);
	}
	_ob_up -> postUnmarshal();
        Boolean _ob_r = _is_a(_ob_a0);
	OutputStreamImpl* out = _ob_up -> preMarshal();
	try
	{
	    out -> write_boolean(_ob_r);
	}
	catch(const CORBA::SystemException& _ob_ex)
	{
	    _ob_up -> marshalEx(_ob_ex);
	}
	_ob_up -> postMarshal();
	return;
    }

    case 2: // _non_existent
    {
        _ob_up -> preUnmarshal();
        _ob_up -> postUnmarshal();
        Boolean _ob_r = _non_existent();
        OutputStreamImpl* out = _ob_up -> preMarshal();
        try
        {
            out -> write_boolean(_ob_r);
        }
        catch(const CORBA::SystemException& _ob_ex)
        {
            _ob_up -> marshalEx(_ob_ex);
        }
        _ob_up -> postMarshal();
        return;
    }

    }

    throw BAD_OPERATION(0, COMPLETED_NO);
}

Long
PortableServer::ServantBase::_OB_findOperation(UpcallImpl* up,
					       const char* names[],
					       ULong num)
{
    const char* op = up -> operation();

    Long l = 0;
    Long r = (Long)num;

    while(l < r)
    {
        Long m = (l + r) / 2;
        int res = strcmp(names[m], op);
        if(res == 0)
            return m;
        else if(res > 0)
            r = m;
        else
            l = m + 1;
    }

    return -1;
}

// ----------------------------------------------------------------------
// ServantBase public member implementation
// ----------------------------------------------------------------------

PortableServer::POA_ptr
PortableServer::ServantBase::_default_POA()
{
    //
    // IMPORTANT!!!! Don't use ORB_init() anywhere else. It is only
    // used here, because ServantBase doesn't have an ORB associated
    // with it, so we have to use the "default" ORB.
    //
    int argcDummy = 0;
    char* argvDummy[1];
    argvDummy[0] = 0;
    ORB_var orb = CORBA::ORB_init(argcDummy, argvDummy, "");
    Object_var obj = orb -> resolve_initial_references("RootPOA");
    return POA::_narrow(obj);
}

Boolean
PortableServer::ServantBase::_is_a(const char* logical_type_id)
    throw(SystemException)
{
    return false;
}

Boolean
PortableServer::ServantBase::_non_existent()
    throw(SystemException)
{
    return false;
}
    
void
PortableServer::ServantBase::_OB_dispatch(UpcallImpl* upcall)
{
    _OB_dispatchBase(upcall);
}

Object_ptr
PortableServer::ServantBase::_OB_createReference()
{
    POA_var thePOA = _default_POA();
    Object_var obj = thePOA -> servant_to_reference(this);
    return obj._retn();
}

ULong
PortableServer::ServantBase::_OB_getRefCount()
{
    return 0;
}

bool
PortableServer::ServantBase::_OB_hasNativeTypes()
{
    return false;
}

// ----------------------------------------------------------------------
// RefCountServantBase public member implementation
// ----------------------------------------------------------------------

void
PortableServer::RefCountServantBase::_add_ref()
{
    _OB_incRef();
}

void
PortableServer::RefCountServantBase::_remove_ref()
{
    _OB_decRef();
}

CORBA::ULong
PortableServer::RefCountServantBase::_OB_getRefCount()
{
    return _OB_getRef();
}



