// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/RefCounted.h>
#include <OBE/Object.h>
#include <OBE/Except.h>
#include <OBE/OBPolicies.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(RefCountIOR_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(RefCountIOR_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(RefCountPolicyList_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(RefCountPolicyList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(RefCountProfileInfo_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(RefCountProfileInfo_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// RefCountPolicyList constructor
// ----------------------------------------------------------------------

Short
RefCountPolicyList::getRetry(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() == RETRY_POLICY_ID)
	{
	    RetryPolicy_var policy = RetryPolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> value();
	}
    }

    return RETRY_STRICT;
}

Long
RefCountPolicyList::getConnectTimeout(const PolicyList& policies)
{
    ULong i;
    for(i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == CONNECT_TIMEOUT_POLICY_ID)
        {
            ConnectTimeoutPolicy_var policy =
                ConnectTimeoutPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return (Long)policy -> value();
        }
    }

    //
    // Fall back to TimeoutPolicy
    //
    for(i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == TIMEOUT_POLICY_ID)
        {
            TimeoutPolicy_var policy = TimeoutPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return (Long)policy -> value();
        }
    }

    return -1;
}

Long
RefCountPolicyList::getRequestTimeout(const PolicyList& policies)
{
    ULong i;
    for(i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == REQUEST_TIMEOUT_POLICY_ID)
        {
            RequestTimeoutPolicy_var policy =
                RequestTimeoutPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return (Long)policy -> value();
        }
    }

    //
    // Fall back to TimeoutPolicy
    //
    for(i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == TIMEOUT_POLICY_ID)
        {
            TimeoutPolicy_var policy = TimeoutPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return (Long)policy -> value();
        }
    }

    return -1;
}

// ----------------------------------------------------------------------
// RefCountPolicyList constructor
// ----------------------------------------------------------------------

RefCountPolicyList::RefCountPolicyList(const PolicyList& v)
    : value(v),
      retry(getRetry(v)),
      connectTimeout(getConnectTimeout(v)),
      requestTimeout(getRequestTimeout(v))
{
}
