// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/Object.h>
#include <OBE/Except.h>
#include <OBE/ORBInstance.h>
#include <OBE/OAInterface.h>
#include <OBE/Server.h>

#ifndef HAVE_OBE_NO_COLLOC
#  include <CollocatedServer.h>
#endif // !HAVE_OBE_NO_COLLOC
#include <GIOPServer.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OB::Server_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::Server_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(OB::ServerManager_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::ServerManager_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// Server constructor and destructor
// ----------------------------------------------------------------------

Server::Server()
{
}

Server::~Server()
{
}

// ----------------------------------------------------------------------
// ServerManager constructor/destructor
// ----------------------------------------------------------------------

ServerManager::ServerManager(ORBInstance_ptr orbInstance,
			     const OCI::AcceptorSeq& acceptors,
                             OAInterface_ptr oaInterface)
    : destroy_(false)
{
    //
    // Create a server for each acceptor, and the collocated server
    //
#ifndef HAVE_OBE_NO_COLLOC
    allServers_.length(acceptors.length() + 1);
#else
    allServers_.length(acceptors.length());
#endif // !HAVE_OBE_NO_COLLOC
    ULong i;
    for(i = 0 ; i < acceptors.length() ; i++)
    {
        GIOPServer* server = new GIOPServer(orbInstance, acceptors[i],
					    oaInterface);
        allServers_[i] = server;
    }
#ifndef HAVE_OBE_NO_COLLOC
    collocatedServer_ = new CollocatedServer(oaInterface);
    allServers_[i] = Server::_duplicate(collocatedServer_);
#endif // !HAVE_OBE_NO_COLLOC
}

ServerManager::~ServerManager()
{
    assert(destroy_);
    assert(allServers_.length() == 0);
#ifndef HAVE_OBE_NO_COLLOC
    assert(is_nil(collocatedServer_));
#endif // !HAVE_OBE_NO_COLLOC
}

// ----------------------------------------------------------------------
// ServerManager public member implementation
// ----------------------------------------------------------------------

void
ServerManager::destroy()
{
    JTCSynchronized synchronized(*this);;

    //
    // Don't destroy twice
    //
    if(destroy_)
	return;

    //
    // Set the destroy flag
    //
    destroy_ = true;

    //
    // Destroy all servers
    //
    for(ULong i = 0; i < allServers_.length(); i++)
	allServers_[i] -> destroy();

    allServers_.length(0);
#ifndef HAVE_OBE_NO_COLLOC
    collocatedServer_ = CollocatedServer::_nil();
#endif // !HAVE_OBE_NO_COLLOC
}

void
ServerManager::activate()
{
    JTCSynchronized synchronized(*this);

    for(ULong i = 0; i < allServers_.length(); i++)
        allServers_[i] -> activate();
}

#ifndef HAVE_OBE_NO_COLLOC
CollocatedServer_ptr
ServerManager::getCollocatedServer()
{
    JTCSynchronized synchronized(*this);

    return CollocatedServer::_duplicate(collocatedServer_);
}
#endif // !HAVE_OBE_NO_COLLOC
