// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/MarshalStubImpl.h>
#include <OBE/Stream.h>
#include <OBE/Principal.h>
#include <OBE/ORBInstance.h>
#include <OBE/RefCounted.h>

#include <stdio.h>
#include <stdlib.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// OutputStreamImpl private and protected member implementations
// ----------------------------------------------------------------------

//
// void
// OutputStreamImpl::writeTypeCodeImpl(TypeCode_ptr tc, WriteTCHistory& history)
//
// In StreamAny.cpp
//

ULong
OutputStreamImpl::writeGap()
{
    addCapacity(4, 4);
    ULong result = buf_.cur_ - buf_.data_;
    buf_.cur_ += 4;
    return result;
}

void
OutputStreamImpl::writeLength(ULong start)
{
    ULong length = buf_.cur_ - buf_.data_ - (start + 4);
    Octet* p = buf_.data_ + start;
    *((ULong*)p) = length;
}

// ----------------------------------------------------------------------
// InputStreamImpl private and protected member implementations
// ----------------------------------------------------------------------

//
// TypeCode_ptr
// InputStreamImpl::readTypeCodeImpl(ReadTCHistory& history)
//
// In StreamAny.cpp
//

void
InputStreamImpl::readShortArray(Short*& value, ULong length, bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

        register Octet* newCur = buf_.cur_;
        newCur += ULong((long)newCur) % 2;
        ULong arrsize = length * 2;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadShortArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (Short*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            UShort* v = (UShort*)value;
            while(length-- > 0)
            {
                *v = (*v << 8) | (*v >> 8);
                v++;
            }
        }
    }
}

void
InputStreamImpl::readUShortArray(UShort*& value, ULong length, bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

        register Octet* newCur = buf_.cur_;
        newCur += ULong((long)newCur) % 2;
        ULong arrsize = length * 2;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadUShortArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (UShort*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            while(length-- > 0)
            {
                *value = (*value << 8) | (*value >> 8);
                value++;
            }
        }
    }
}


void
InputStreamImpl::readLongArray(Long*& value, ULong length, bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

        register Octet* newCur = buf_.cur_ + 3;
        newCur -= ULong((long)newCur) % 4;
        ULong arrsize = length * 4;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadLongArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (Long*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            ULong* v = (ULong*)value;
            while(length-- > 0)
            {
                *v = ((*v << 24) | ((*v & 0xff00) << 8)
                      | ((*v >> 8) & 0xff00) | (*v >> 24));
                v++;
            }
        }
    }
}

void
InputStreamImpl::readULongArray(ULong*& value, ULong length, bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

        register Octet* newCur = buf_.cur_ + 3;
        newCur -= ULong((long)newCur) % 4;
        ULong arrsize = length * 4;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadULongArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (ULong*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            while(length-- > 0)
            {
                *value = ((*value << 24) | ((*value & 0xff00) << 8)
                      | ((*value >> 8) & 0xff00) | (*value >> 24));
                value++;
            }
        }
    }
}

void
InputStreamImpl::readLongLongArray(LongLong*& value, ULong length, bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

        register Octet* newCur = buf_.cur_ + 7;
        newCur -= ULong((long)newCur) % 8;
        ULong arrsize = length * 8;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadLongLongArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (LongLong*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            ULongLong* v = (ULongLong*)value;
            while(length-- > 0)
            {
                *v = ((*v << 56) | ((*v & 0xff00) << 40)
                      | ((*v & 0xff0000) << 24) | ((*v & 0xff000000) << 8)
                      | ((*v >> 8) & 0xff000000) | ((*v >> 24) & 0xff0000)
                      | ((*v >> 40) & 0xff00) | (*v >> 56));
                v++;
            }
        }
    }
}

void
InputStreamImpl::readULongLongArray(ULongLong*& value, ULong length, bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

        register Octet* newCur = buf_.cur_ + 7;
        newCur -= ULong((long)newCur) % 8;
        ULong arrsize = length * 8;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadULongLongArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (ULongLong*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            ULongLong* v = value;
            while(length-- > 0)
            {
                *v = ((*v << 56) | ((*v & 0xff00) << 40)
                      | ((*v & 0xff0000) << 24) | ((*v & 0xff000000) << 8)
                      | ((*v >> 8) & 0xff000000) | ((*v >> 24) & 0xff0000)
                      | ((*v >> 40) & 0xff00) | (*v >> 56));
                v++;
            }
        }
    }
}


void
InputStreamImpl::readFloatArray(Float*& value, ULong length, bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

        register Octet* newCur = buf_.cur_ + 3;
        newCur -= ULong((long)newCur) % 4;
        ULong arrsize = length * 4;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadFloatArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (Float*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            ULong* v = (ULong*)value;
            while(length-- > 0)
            {
                *v = ((*v << 24) | ((*v & 0xff00) << 8)
                      | ((*v >> 8) & 0xff00) | (*v >> 24));
                v++;
            }
        }
    }
}

void
InputStreamImpl::readDoubleArray(Double*& value, ULong length, bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

        register Octet* newCur = buf_.cur_ + 7;
        newCur -= ULong((long)newCur) % 8;
        ULong arrsize = length * 8;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadDoubleArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (Double*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            ULong* v = (ULong*)value;
            while(length-- > 0)
            {
                ULong v0 = v[0];
                ULong v1 = v[1];
                v0 = ((v0 << 24) | ((v0 & 0xff00) << 8)
                      | ((v0 >> 8) & 0xff00) | (v0 >> 24));
                v1 = ((v1 << 24) | ((v1 & 0xff00) << 8)
                      | ((v1 >> 8) & 0xff00) | (v1 >> 24));
                v[0] = v1;
                v[1] = v0;
                v += 2;
            }
        }
    }
}


#if SIZEOF_LONG_DOUBLE < 12

void
InputStreamImpl::readLongDoubleArray(LongDouble*&, ULong, bool)
{
    throw MARSHAL(MinorLongDoubleNotSupported, COMPLETED_NO);
}

#else

void
InputStreamImpl::readLongDoubleArray(LongDouble*& value, ULong length,
				     bool copy)
{
    if(length)
    {
        assert_nca(!copy || (copy && value != 0), NCANullValue);

#if SIZEOF_LONG_DOUBLE == 16

        register Octet* newCur = buf_.cur_ + 7;
        newCur -= ULong((long)newCur) % 8;
        ULong arrsize = length * 16;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur > buf_.last_)
            throw MARSHAL(MinorReadLongDoubleArrayOverflow, COMPLETED_NO);

	if(copy)
	    memcpy(value, newCur, arrsize);
	else
	    value = (LongDouble*)newCur;
        buf_.cur_ = nextCur;

        if(swap_)
        {
            ULongLong* v = (ULongLong*)value;
            while(length-- > 0)
            {
                ULongLong v0 = v[0];
                ULongLong v1 = v[1];
                v0 = ((v0 << 56) | ((v0 & 0xff00) << 40)
                      | ((v0 & 0xff0000) << 24) | ((v0 & 0xff000000) << 8)
                      | ((v0 >> 8) & 0xff000000) | ((v0 >> 24) & 0xff0000)
                      | ((v0 >> 40) & 0xff00) | (v0 >> 56));
                v1 = ((v1 << 56) | ((v1 & 0xff00) << 40)
                      | ((v1 & 0xff0000) << 24) | ((v1 & 0xff000000) << 8)
                      | ((v1 >> 8) & 0xff000000) | ((v1 >> 24) & 0xff0000)
                      | ((v1 >> 40) & 0xff00) | (v1 >> 56));
                v[0] = v1;
                v[1] = v0;
                v += 2;
            }
        }

#endif

#if SIZEOF_LONG_DOUBLE == 12

	// XXX
        ULong i;
        for(i = 0 ; i < length ; i++)
            value[i] = read_longdouble();

#endif
    }
}

#endif
// ----------------------------------------------------------------------
// InputStreamImpl constructors and destructor
// ----------------------------------------------------------------------

InputStreamImpl::InputStreamImpl(ULong len, ULong offs, bool swap)
    : orbInstance_(ORBInstance::_nil()),
      buf_(len), origSwap_(swap), swap_(swap)
{
    buf_.cur_ = buf_.data_ + offs;
    origCur_ = buf_.cur_;
}

InputStreamImpl::InputStreamImpl(Octet* data, ULong len)
    : orbInstance_(ORBInstance::_nil()),
      buf_(data, len), origSwap_(false), swap_(false)
{
    origCur_ = buf_.cur_;
}

// ----------------------------------------------------------------------
// InputStreamImpl public member implementations
// ----------------------------------------------------------------------

InputStreamImpl*
InputStreamImpl::_OB_clone()
{
    InputStreamImpl* result =
	new InputStreamImpl(buf_.len_, origCur_ - buf_.data_, origSwap_);
    memcpy(result -> _OB_bufferImpl() -> data_, buf_.data_, buf_.len_);
    result -> orbInstance_ = orbInstance_;
    return result;
}

void
InputStreamImpl::_OB_reset()
{
    buf_.cur_ = origCur_;
    swap_ = origSwap_;
}

ORBInstance_ptr
InputStreamImpl::_OB_ORBInstance_nodup()
{
    return orbInstance_;
}

//
// Note that we do not duplicate ORBInstance since we know that it
// will exist until the ORB is destroyed and thus will not go away
// from underneath us making the pointer invalid.
//
void
InputStreamImpl::_OB_ORBInstance(ORBInstance_ptr orbInstance)
{
    orbInstance_ = orbInstance;
}

void
InputStreamImpl::_OB_readEndian()
{
    swap_ = (read_boolean() != Endian);
}

Long
InputStreamImpl::_OB_readLongUnchecked()
{
    //
    // The chunking code needs to read a long value without entering
    // an infinite loop
    //

    Long val;

    register Octet* newCur = buf_.cur_ + 3;
    newCur -= ULong((long)newCur) % 4;

    register Octet* nextCur = newCur + 4;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadLongOverflow, COMPLETED_NO);

    val = *(const Long*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
    {
        ULong& v = (ULong&)val;
        v = ((v << 24) | ((v & 0xff00) << 8)
             | ((v >> 8) & 0xff00) | (v >> 24));
    }

    return val;
}

Char
InputStreamImpl::read_char()
{
    if(buf_.cur_ > buf_.last_)
        throw MARSHAL(MinorReadCharOverflow, COMPLETED_NO);

    Char value = *(buf_.cur_);
    buf_.cur_++;
    return value;
}

Short
InputStreamImpl::read_short()
{
    Short val;

    register Octet* newCur = buf_.cur_;
    newCur += ULong((long)newCur) % 2;

    register Octet* nextCur = newCur + 2;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadShortOverflow, COMPLETED_NO);

    val = *(const Short*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
    {
        UShort& v = (UShort&)val;
        v = (v << 8) | (v >> 8);
    }

    return val;
}

UShort
InputStreamImpl::read_ushort()
{
    UShort val;

    register Octet* newCur = buf_.cur_;
    newCur += ULong((long)newCur) % 2;

    register Octet* nextCur = newCur + 2;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadUShortOverflow, COMPLETED_NO);

    val = *(const UShort*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
        val = (val << 8) | (val >> 8);

    return val;
}

Long
InputStreamImpl::read_long()
{
    return _OB_readLongUnchecked();
}

ULong
InputStreamImpl::read_ulong()
{
    ULong val;

    register Octet* newCur = buf_.cur_ + 3;
    newCur -= ULong((long)newCur) % 4;

    register Octet* nextCur = newCur + 4;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadULongOverflow, COMPLETED_NO);

    val = *(const ULong*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
        val = ((val << 24) | ((val & 0xff00) << 8)
               | ((val >> 8) & 0xff00) | (val >> 24));

    return val;
}

LongLong
InputStreamImpl::read_longlong()
{
    LongLong val;

    register Octet* newCur = buf_.cur_ + 7;
    newCur -= ULong((long)newCur) % 8;

    register Octet* nextCur = newCur + 8;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadLongLongOverflow, COMPLETED_NO);

    val = *(const LongLong*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
    {
	ULongLong& v = (ULongLong&)val;
	v = ((v << 56) | ((v & 0xff00) << 40)
	     | ((v & 0xff0000) << 24) | ((v & 0xff000000) << 8)
	     | ((v >> 8) & 0xff000000) | ((v >> 24) & 0xff0000)
	     | ((v >> 40) & 0xff00) | (v >> 56));
    }

    return val;
}

ULongLong
InputStreamImpl::read_ulonglong()
{
    ULongLong val;

    register Octet* newCur = buf_.cur_ + 7;
    newCur -= ULong((long)newCur) % 8;

    register Octet* nextCur = newCur + 8;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadULongLongOverflow, COMPLETED_NO);

    val = *(const ULongLong*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
        val = ((val << 56) | ((val & 0xff00) << 40)
               | ((val & 0xff0000) << 24) | ((val & 0xff000000) << 8)
               | ((val >> 8) & 0xff000000) | ((val >> 24) & 0xff0000)
               | ((val >> 40) & 0xff00) | (val >> 56));

    return val;
}

Float
InputStreamImpl::read_float()
{
    Float val;

    register Octet* newCur = buf_.cur_ + 3;
    newCur -= ULong((long)newCur) % 4;

    register Octet* nextCur = newCur + 4;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadFloatOverflow, COMPLETED_NO);

    val = *(const Float*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
    {
        ULong* v = (ULong*)&val;
        *v = ((*v << 24) | ((*v & 0xff00) << 8)
              | ((*v >> 8) & 0xff00) | (*v >> 24));
    }

    return val;
}

Double
InputStreamImpl::read_double()
{
    Double val;

    register Octet* newCur = buf_.cur_ + 7;
    newCur -= ULong((long)newCur) % 8;

    register Octet* nextCur = newCur + 8;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadDoubleOverflow, COMPLETED_NO);

    val = *(const Double*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
    {
        ULong v0 = ((ULong*)&val)[0];
        ULong v1 = ((ULong*)&val)[1];
        v0 = ((v0 << 24) | ((v0 & 0xff00) << 8)
              | ((v0 >> 8) & 0xff00) | (v0 >> 24));
        v1 = ((v1 << 24) | ((v1 & 0xff00) << 8)
              | ((v1 >> 8) & 0xff00) | (v1 >> 24));
        ((ULong*)&val)[0] = v1;
        ((ULong*)&val)[1] = v0;
    }

    return val;
}

#if SIZEOF_LONG_DOUBLE < 12

LongDouble
InputStreamImpl::read_longdouble()
{
    throw MARSHAL(MinorLongDoubleNotSupported, COMPLETED_NO);
    return LongDouble(); // Some compilers need this
}

#else

LongDouble
InputStreamImpl::read_longdouble()
{
    LongDouble val;

    register Octet* newCur = buf_.cur_ + 7;
    newCur -= ULong((long)newCur) % 8;

    register Octet* nextCur = newCur + 16;
    if(nextCur > buf_.last_)
        throw MARSHAL(MinorReadLongDoubleOverflow, COMPLETED_NO);

#if SIZEOF_LONG_DOUBLE == 16

    val = *(const LongDouble*)newCur;
    buf_.cur_ = nextCur;

    if(swap_)
    {
        ULongLong v0 = ((ULongLong*)&val)[0];
        ULongLong v1 = ((ULongLong*)&val)[1];
        v0 = ((v0 << 56) | ((v0 & 0xff00) << 40)
              | ((v0 & 0xff0000) << 24) | ((v0 & 0xff000000) << 8)
              | ((v0 >> 8) & 0xff000000) | ((v0 >> 24) & 0xff0000)
              | ((v0 >> 40) & 0xff00) | (v0 >> 56));
        v1 = ((v1 << 56) | ((v1 & 0xff00) << 40)
              | ((v1 & 0xff0000) << 24) | ((v1 & 0xff000000) << 8)
              | ((v1 >> 8) & 0xff000000) | ((v1 >> 24) & 0xff0000)
              | ((v1 >> 40) & 0xff00) | (v1 >> 56));
        ((ULongLong*)&val)[0] = v1;
        ((ULongLong*)&val)[1] = v0;
    }

#endif

#if SIZEOF_LONG_DOUBLE == 12
#   ifdef WORDS_BIGENDIAN

#       error 12-bytes long doubles only supported for little endian!

#   else

    Octet tmp[18];

    if(swap_)
    {
	*(UShort*)&tmp = 0;
	memcpy(tmp + 2, newCur, 14);

        ULongLong v0 = ((ULongLong*)&tmp)[0];
        ULongLong v1 = ((ULongLong*)&tmp)[1];
        v0 = ((v0 << 56) | ((v0 & 0xff00) << 40)
              | ((v0 & 0xff0000) << 24) | ((v0 & 0xff000000) << 8)
              | ((v0 >> 8) & 0xff000000) | ((v0 >> 24) & 0xff0000)
              | ((v0 >> 40) & 0xff00) | (v0 >> 56));
        v1 = ((v1 << 56) | ((v1 & 0xff00) << 40)
              | ((v1 & 0xff0000) << 24) | ((v1 & 0xff000000) << 8)
              | ((v1 >> 8) & 0xff000000) | ((v1 >> 24) & 0xff0000)
              | ((v1 >> 40) & 0xff00) | (v1 >> 56));
        ((ULongLong*)&tmp)[0] = v1;
        ((ULongLong*)&tmp)[1] = v0;

	*(ULongLong*)(tmp + 4) >>= 1; // Adjust mantissa
	tmp[11] |= 0x80;              // Set leading significant bit

	val = *(const LongDouble*)(tmp + 4);
    }
    else
    {
	*(UShort*)&tmp[16] = 0;
	memcpy(tmp + 2, newCur + 2, 14);

	*(ULongLong*)(tmp + 6) >>= 1; // Adjust mantissa
	tmp[13] |= 0x80;              // Set leading significant bit

	val = *(const LongDouble*)(tmp + 6);
    }

    buf_.cur_ = nextCur;

#   endif

#endif

    return val;
}

#endif

char*
InputStreamImpl::read_string()
{
    char* s;

    ULong len = read_ulong();
    if(len == 0)
        throw MARSHAL(MinorReadStringZeroLength, COMPLETED_NO);

    if(buf_.cur_ + len > buf_.last_)
        throw MARSHAL(MinorReadStringOverflow, COMPLETED_NO);

    s = string_alloc(len - 1);
    memcpy(s, buf_.cur_, len - 1);
    buf_.cur_ += len - 1;

    //
    // Check for terminating null char
    //
    if(*(buf_.cur_) != '\0')
    {
        string_free(s);
	throw MARSHAL(MinorReadStringNoTerminator, COMPLETED_NO);
    }

    s[len - 1] = '\0';
    buf_.cur_++;

    if(strlen(s) + 1 != len)
    {
        string_free(s);
        throw MARSHAL(MinorReadStringNullChar, COMPLETED_NO);
    }

    return s;
}

const char*
InputStreamImpl::read_string_nodup()
{
    ULong len = read_ulong();
    if(len == 0)
        throw MARSHAL(MinorReadStringZeroLength, COMPLETED_NO);

    if(buf_.cur_ + len > buf_.last_)
        throw MARSHAL(MinorReadStringOverflow, COMPLETED_NO);

    const char* s = (const char*)buf_.cur_;
    buf_.cur_ += len;

    return s;
}

Object_ptr
InputStreamImpl::read_Object()
{
    IOP::IOR ior;
    IOP::IOR::_OB_unmarshal(ior, this);

    if(strlen(ior.type_id) == 0 && ior.profiles.length() == 0)
        return Object::_nil();
    else
    {
        assert(!is_nil(orbInstance_));
        ObjectFactory_var objectFactory = orbInstance_ -> getObjectFactory();
        RefCountIOR_var refIOR = new RefCountIOR(ior);
        return objectFactory -> createObject(refIOR);
    }
}

//
// TypeCode_ptr
// InputStreamImpl::read_TypeCode()
//
// In StreamAny.cpp
//

Principal_ptr
InputStreamImpl::read_Principal()
{
    //
    // Read a sequence of octets
    //
    ULong len = read_ulong();
    _OB_skip(len);

    return Principal::_nil();
}

//
// Any*
// InputStreamImpl::read_any()
//
// void
// InputStreamImpl::read_any(Any& any)
//
// In StreamAny.cpp
//

//
// void
// InputStreamImpl::read_any_NoTypeCode(Any& any, TypeCode_ptr tc)
//
// In StreamAny.cpp
//

// ----------------------------------------------------------------------
// OutputStreamImpl constructors and destructor
// ----------------------------------------------------------------------

OutputStreamImpl::OutputStreamImpl(ULong len)
    : buf_(len),
      alignNext_(0)
{
}

OutputStreamImpl::OutputStreamImpl()
    : alignNext_(0)
{
}

// ----------------------------------------------------------------------
// OutputStreamImpl public member implementations
// ----------------------------------------------------------------------

InputStreamImpl*
OutputStreamImpl::create_input_stream()
{
    InputStreamImpl* result = new InputStreamImpl(buf_.len_, 0, false);
    memcpy(result -> _OB_bufferImpl() -> data_, buf_.data_, buf_.len_);
    return result;
}

void
OutputStreamImpl::write_char(Char value)
{
    addCapacity(1);

    *(buf_.cur_) = (Octet)value;
    buf_.cur_++;
}

void
OutputStreamImpl::write_char_array(const Char* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);
	addCapacity(length);

        memcpy(buf_.cur_, value, length);
        buf_.cur_ += length;
    }
}

#if SIZEOF_LONG_DOUBLE < 12

void
OutputStreamImpl::write_longdouble(LongDouble value)
{
    throw MARSHAL(MinorLongDoubleNotSupported, COMPLETED_NO);
}

#else

void
OutputStreamImpl::write_longdouble(LongDouble value)
{
    addCapacity(16, 8);

#if SIZEOF_LONG_DOUBLE == 16

    *(LongDouble*)(buf_.cur_) = value;

#endif

#if SIZEOF_LONG_DOUBLE == 12

#   ifdef WORDS_BIGENDIAN

#       error 12-bytes long doubles only supported for little endian!

#   else

    register Octet* cur = buf_.cur_;
    *(ULongLong*)cur = 0;
    *(LongDouble*)(cur + 6) = value;
    *(cur + 13) &= 0x7f; // Clear leading significant bit
    *(ULongLong*)(cur + 6) <<= 1; // Adjust mantissa

#   endif

#endif

    buf_.cur_ += 16;
}

#endif

void
OutputStreamImpl::write_string(const char* value)
{
    assert_nca(value != 0, NCANullString);
    ULong len = strlen(value) + 1;
    write_ulong(len);

    addCapacity(len);

    memcpy(buf_.cur_, value, len - 1);
    buf_.cur_ += len - 1;

    *(buf_.cur_) = '\0';
    buf_.cur_++;
}

void
OutputStreamImpl::write_Object(Object_ptr value)
{
    if(is_nil(value))
    {
        IOP::IOR ior;
        ior.type_id = string_dup("");
        ior._OB_marshal(this);
    }
    else
    {
        if(value -> _OB_isLC())
            throw MARSHAL(MinorWriteObjectLocal, COMPLETED_NO);

	value -> _OB_marshalOrigIOR(this);
    }
}

//
// void
// OutputStreamImpl::write_TypeCode(TypeCode_ptr tc)
//
// In StreamAny.cpp
//

void
OutputStreamImpl::write_Principal(Principal_ptr)
{
    //
    // Just marshal an empty sequence of octets
    //
    write_ulong(0);
}

//
// void
// OutputStreamImpl::write_any(const Any& value)
//
// In StreamAny.cpp
//

#if SIZEOF_LONG_DOUBLE < 12

void
OutputStreamImpl::write_longdouble_array(const LongDouble* value, ULong length)
{
    throw MARSHAL(MinorLongDoubleNotSupported, COMPLETED_NO);
}

#else

void
OutputStreamImpl::write_longdouble_array(const LongDouble* value, ULong length)
{
    if(length)
    {
#if SIZEOF_LONG_DOUBLE == 16

        ULong arrsize = length * 16;
        addCapacity(arrsize, 8);
        memcpy(buf_.cur_, value, arrsize);
        buf_.cur_ += arrsize;

#endif

#if SIZEOF_LONG_DOUBLE == 12

        ULong i;
        for(i = 0 ; i < length ; i++)
            write_longdouble(value[i]);

#endif
    }
}

#endif

//
// void
// OutputStreamImpl::write_any_NoTypeCode(const Any& any)
//
// void
// OutputStreamImpl::write_InputStreamImpl(InputStreamImpl* in, TypeCode_ptr tc)
//
// In StreamAny.cpp
//
