// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/TypeCode.h>
#include <OBE/Principal.h>
#include <OBE/MarshalStubImpl.h>
#include <OBE/Stream.h>
#include <OBE/ORBInstance.h>
#include <OBE/RefCounted.h>

#include <stdio.h>
#include <stdlib.h>

using namespace CORBA;
using namespace OB;

namespace OB
{

struct OctetPtrHasher
{
    static CORBA::ULong
    hash(const CORBA::Octet* k)
    {
	return (CORBA::ULong)((long)k);
    }

    static bool comp(const CORBA::Octet* k1, const CORBA::Octet* k2)
    {
	return k1 == k2;
    }
};
typedef Hashtable<const CORBA::Octet*, CORBA::TypeCode_ptr,
		  OctetPtrHasher> ReadTCHistory;

struct TypeCodePtrHasher
{
    static CORBA::ULong
    hash(const CORBA::TypeCode_ptr k)
    {
	return (CORBA::ULong)((long)k);
    }

    static bool
    comp(const CORBA::TypeCode_ptr k1, const CORBA::TypeCode_ptr k2)
    {
	return k1 == k2;
    }
};
typedef Hashtable<CORBA::TypeCode_ptr, CORBA::Long, 
    TypeCodePtrHasher> WriteTCHistory;

class TypeCodeReader
{
public:
    static CORBA::TypeCode_ptr read(InputStreamImpl*, ReadTCHistory&);
};

class TypeCodeWriter
{
public:
    static void write(OutputStreamImpl*, CORBA::TypeCode_ptr, WriteTCHistory&);
};

} // End of namespace OB


// ----------------------------------------------------------------------
// TypeCodeWriter public member implementations
// ----------------------------------------------------------------------

void
TypeCodeWriter::write(OutputStreamImpl* impl, TypeCode_ptr tc,
		      WriteTCHistory& history)
{
    if(tc -> recId_.in())
    {
        assert_nca(!is_nil(tc -> recType_), NCAUnembeddedRecTC);
        write(impl, tc -> recType_, history);
        return;
    }

    Long indirectionPos;
    if(history.get(tc, indirectionPos))
    {
        impl -> write_ulong(0xffffffff);
        Long offs = indirectionPos -
	    (Long)(impl -> buf_.cur_ - impl -> buf_.data_);
        impl -> write_long(offs);
    }
    else
    {
        impl -> write_ulong((ULong)tc -> kind());
        Long oldPos = (Long)(impl -> buf_.cur_ - impl -> buf_.data_ - 4);

        switch(tc -> kind())
        {
        case tk_null:
        case tk_void:
        case tk_short:
        case tk_long:
        case tk_longlong:
        case tk_ushort:
        case tk_ulong:
        case tk_ulonglong:
        case tk_float:
        case tk_double:
        case tk_longdouble:
        case tk_boolean:
        case tk_char:
        case tk_octet:
        case tk_any:
        case tk_TypeCode:
        case tk_Principal:
            break;

        case tk_objref:
        {
            history.put(tc, oldPos);

            ULong start = impl -> writeGap();
            impl -> _OB_writeEndian();
            impl -> write_string(tc -> id_);
            impl -> write_string(tc -> name_);
            impl -> writeLength(start);

            break;
        }

        case tk_struct:
        case tk_except:
        {
            history.put(tc, oldPos);

            ULong i;

            ULong start = impl -> writeGap();
            impl -> _OB_writeEndian();
            impl -> write_string(tc -> id_);
            impl -> write_string(tc -> name_);
            impl -> write_ulong(tc -> memberNames_.length());
            for(i = 0 ; i < tc -> memberNames_.length() ; i++)
            {
                impl -> write_string(tc -> memberNames_[i]);
                write(impl, tc -> memberTypes_[i], history);
            }
            impl -> writeLength(start);

            break;
        }

        case tk_union:
        {
            history.put(tc, oldPos);

            ULong start = impl -> writeGap();
            impl -> _OB_writeEndian();
            impl -> write_string(tc -> id_);
            impl -> write_string(tc -> name_);
            write(impl, tc -> discriminatorType_, history);
            Long defaultIndex = tc -> _OB_defaultIndex();
            impl -> write_long(defaultIndex);
            impl -> write_ulong(tc -> memberNames_.length());
            for(ULong i = 0 ; i < tc -> memberNames_.length() ; i++)
            {
                //
                // Check for default label value
                //
                if(i == (ULong)defaultIndex)
                {
                    //
                    // Marshal a dummy value of the appropriate size
                    // for the discriminator type
                    //
                    TypeCode_ptr origDiscType =
                        GetOrigType_nodup(tc -> discriminatorType_);
                    switch(origDiscType -> kind())
                    {
                    case tk_short:
                        impl -> write_short(0);
                        break;
                    case tk_ushort:
                        impl -> write_ushort(0);
                        break;
                    case tk_long:
                        impl -> write_long(0);
                        break;
                    case tk_ulong:
                        impl -> write_ulong(0);
                        break;
                    case tk_longlong:
                        impl -> write_longlong(0);
                        break;
                    case tk_ulonglong:
                        impl -> write_ulonglong(0);
                        break;
                    case tk_boolean:
                        impl -> write_boolean(false);
                        break;
                    case tk_char:
                        impl -> write_char(0);
                        break;
                    case tk_enum:
                        impl -> write_ulong(0);
                        break;
                    default:
                        assert(false);
                        break;
                    }
                }
                else
                {
                    impl -> write_any_NoTypeCode(tc -> labels_[i]);
                }

                impl -> write_string(tc -> memberNames_[i]);
                write(impl, tc -> memberTypes_[i], history);
            }
            impl -> writeLength(start);

            break;
        }

        case tk_enum:
        {
            history.put(tc, oldPos);

            ULong start = impl -> writeGap();
            impl -> _OB_writeEndian();
            impl -> write_string(tc -> id_);
            impl -> write_string(tc -> name_);
            impl -> write_ulong(tc -> memberNames_.length());
            for(ULong i = 0 ; i < tc -> memberNames_.length() ; i++)
                impl -> write_string(tc -> memberNames_[i]);
            impl -> writeLength(start);

            break;
        }

        case tk_string:
            impl -> write_ulong(tc -> length_);
            break;

        case tk_sequence:
        case tk_array:
        {
            history.put(tc, oldPos);

            ULong start = impl -> writeGap();
            impl -> _OB_writeEndian();
            write(impl, tc -> contentType_, history);
            impl -> write_ulong(tc -> length_);
            impl -> writeLength(start);

            break;
        }

        case tk_alias:
        {
            history.put(tc, oldPos);

            ULong start = impl -> writeGap();
            impl -> _OB_writeEndian();
            impl -> write_string(tc -> id_);
            impl -> write_string(tc -> name_);
            write(impl, tc -> contentType_, history);
            impl -> writeLength(start);

            break;
        }

        case tk_native:
        {
            history.put(tc, oldPos);

            ULong start = impl -> writeGap();
            impl -> _OB_writeEndian();
            impl -> write_string(tc -> id_);
            impl -> write_string(tc -> name_);
            impl -> writeLength(start);

            break;
        }

	//
	// These are not supported in OBE
	//
        case tk_fixed:
        case tk_wchar:
        case tk_wstring:
        case tk_value:
        case tk_value_box:
        case tk_abstract_interface:
	default:
	    assert(false);
	    break; // this is to keep the compiler happy
        }
    }
}

// ----------------------------------------------------------------------
// TypeCodeReader public member implementations
// ----------------------------------------------------------------------

TypeCode_ptr
TypeCodeReader::read(InputStreamImpl* impl, ReadTCHistory& history)
{
    TCKind kind = (TCKind)impl -> read_ulong();
    Octet* oldPos = impl -> buf_.cur_ - 4;

    TypeCode_var tc;
    if(kind == (TCKind)0xffffffff)
    {
        Long offs = impl -> read_long();
        Octet* indirectionPos = impl -> buf_.cur_ - 4 + offs;
        // Adjust for alignment
        indirectionPos += ULong((long)indirectionPos) % 4;
        TypeCode_ptr p;
        if(!history.get(indirectionPos, p))
            throw MARSHAL(MinorReadInvTypeCodeIndirection, COMPLETED_NO);
        history.put(oldPos, p);
        tc = TypeCode::_duplicate(p);
    }
    else
    {
        switch(kind)
        {
        case tk_null:
        case tk_void:
        case tk_short:
        case tk_long:
        case tk_longlong:
        case tk_ushort:
        case tk_ulong:
        case tk_ulonglong:
        case tk_float:
        case tk_double:
        case tk_longdouble:
        case tk_boolean:
        case tk_char:
        case tk_octet:
        case tk_any:
        case tk_TypeCode:
        case tk_Principal:
        {
            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            break;
        }

        case tk_objref:
        {
            impl -> read_ulong(); // encapsulation length
            bool swap = impl -> swap_;
            impl -> _OB_readEndian();

            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> id_ = impl -> read_string();
            tc -> name_ = impl -> read_string();

            impl -> swap_ = swap;
            break;
        }

        case tk_struct:
        case tk_except:
        {
            impl -> read_ulong(); // encapsulation length
            bool swap = impl -> swap_;
            impl -> _OB_readEndian();

            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> id_ = impl -> read_string();
            tc -> name_ = impl -> read_string();
            ULong num = impl -> read_ulong();
            tc -> memberNames_.length(num);
            tc -> memberTypes_.length(num);
            for(ULong i = 0 ; i < num ; i++)
            {
                tc -> memberNames_[i] = impl -> read_string();
                tc -> memberTypes_[i] = read(impl, history);
            }

            impl -> swap_ = swap;
            break;
        }

        case tk_union:
        {
            impl -> read_ulong(); // encapsulation length
            bool swap = impl -> swap_;
            impl -> _OB_readEndian();

            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> id_ = impl -> read_string();
            tc -> name_ = impl -> read_string();
            tc -> discriminatorType_ = read(impl, history);
            Long defaultIndex = impl -> read_long();
            ULong num = impl -> read_ulong();
            tc -> labels_.length(num);
            tc -> memberNames_.length(num);
            tc -> memberTypes_.length(num);
            for(ULong i = 0 ; i < num ; i++)
            {
                if((Long)i == defaultIndex)
                {
                    //
                    // Unmarshal a dummy value of the appropriate size
                    // for the discriminator type
                    //
                    Any dummy;
                    impl -> read_any_NoTypeCode(dummy,
						tc -> discriminatorType_);

                    //
                    // Default label value is the zero octet
                    //
                    tc -> labels_[i] <<= Any::from_octet(0);
                }
                else
                    impl -> read_any_NoTypeCode(tc -> labels_[i],
						tc -> discriminatorType_);

                tc -> memberNames_[i] = impl -> read_string();
                tc -> memberTypes_[i] = read(impl, history);
            }

            impl -> swap_ = swap;
            break;
        }

        case tk_enum:
        {
            impl -> read_ulong(); // encapsulation length
            bool swap = impl -> swap_;
            impl -> _OB_readEndian();

            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> id_ = impl -> read_string();
            tc -> name_ = impl -> read_string();
            ULong num = impl -> read_ulong();
            tc -> memberNames_.length(num);
            for(ULong i = 0 ; i < num ; i++)
                tc -> memberNames_[i] = impl -> read_string();

            impl -> swap_ = swap;
            break;
        }

        case tk_string:
        {
            ULong bound = impl -> read_ulong();
            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> length_ = bound;
            break;
        }

        case tk_sequence:
        case tk_array:
        {
            impl -> read_ulong(); // encapsulation length
            bool swap = impl -> swap_;
            impl -> _OB_readEndian();

            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> contentType_ = read(impl, history);
            tc -> length_ = impl -> read_ulong();

            impl -> swap_ = swap;
            break;
        }

        case tk_alias:
        {
            impl -> read_ulong(); // encapsulation length
            bool swap = impl -> swap_;
            impl -> _OB_readEndian();

            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> id_ = impl -> read_string();
            tc -> name_ = impl -> read_string();
            tc -> contentType_ = read(impl, history);

            impl -> swap_ = swap;
            break;
        }

        case tk_native:
        {
            impl -> read_ulong(); // encapsulation length
            bool swap = impl -> swap_;
            impl -> _OB_readEndian();

            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> id_ = impl -> read_string();
            tc -> name_ = impl -> read_string();

            impl -> swap_ = swap;
            break;
        }

	//
	// These are not supported in OBE
	//
        case tk_fixed:
        case tk_wchar:
        case tk_wstring:
        case tk_value:
        case tk_value_box:
        case tk_abstract_interface:
        default:
            throw MARSHAL(); // unknown kind
	    break; // this is to keep the compiler happy
        }
    }

    assert(!is_nil(tc));
    return tc._retn();
}

// ----------------------------------------------------------------------
// InputStreamImpl public member implementations
// ----------------------------------------------------------------------

TypeCode_ptr
InputStreamImpl::read_TypeCode()
{
    //
    // NOTE:
    //
    // No data with natural alignment of greater than four octets is
    // needed for TypeCode. Therefore it is not necessary to do
    // encapsulation in a seperate buffer.
    //

    ReadTCHistory history(11);
    return TypeCodeReader::read(this, history);
}

Any*
InputStreamImpl::read_any()
{
    TypeCode_var tc = read_TypeCode();
    Any_var any = new Any;
    read_any_NoTypeCode(any.inout(), tc);
    return any._retn();
}

// 
// This version of read_any is needed to avoid excessive copying
// of inout parameters
//
void
InputStreamImpl::read_any(Any& any)
{
    TypeCode_var tc = read_TypeCode();
    read_any_NoTypeCode(any, tc);
}

void
InputStreamImpl::read_any_NoTypeCode(Any& any, TypeCode_ptr tc)
{
    any._OB_ORBInstance(orbInstance_);

    void* val = 0;
    
    TypeCode_ptr p = GetOrigType_nodup(tc);
    
    switch(p -> kind())
    {
    case tk_null:
    case tk_void:
	break;
	
    case tk_short:
    {
	Short v = read_short();
	val = new Short(v);
	break;
    }

    case tk_long:
    {
	Long v = read_long();
	val = new Long(v);
	break;
    }

    case tk_longlong:
    {
	LongLong v = read_longlong();
	val = new LongLong(v);
	break;
     }

    case tk_ushort:
    {
	UShort v = read_ushort();
	val = new UShort(v);
	break;
    }

    case tk_ulong:
    case tk_enum:
    {
	ULong v = read_ulong();
	val = new ULong(v);
	break;
    }

    case tk_ulonglong:
    {
	ULongLong v = read_ulonglong();
	val = new ULongLong(v);
	break;
    }

    case tk_float:
    {
	Float v = read_float();
	val = new Float(v);
	break;
    }

    case tk_double:
    {
	Double v = read_double();
	val = new Double(v);
	break;
    }

    case tk_longdouble:
    {
	LongDouble v = read_longdouble();
	val = new LongDouble(v);
	break;
    }

    case tk_boolean:
    {
	Boolean v = read_boolean();
	val = new Boolean(v);
	break;
    }

    case tk_char:
    {
	Char v = read_char();
	val = new Char(v);
	break;
    }

    case tk_octet:
    {
	Octet v = read_octet();
	val = new Octet(v);
	break;
    }

    case tk_any:
    {
	val = read_any();
	break;
    }

    case tk_TypeCode:
    {
	val = read_TypeCode();
	break;
    }

    case tk_Principal:
    {
	val = read_Principal();
	break;
    }
	
    case tk_objref:
    {
	val = read_Object();
	break;
    }

    case tk_struct:
    case tk_sequence:
    case tk_union:
    case tk_array:
    case tk_except:

	if(p -> _OB_isSystemException())
	{
	    val = SystemException::_OB_unmarshal(this);
	}
	else
	{
	    OutputStreamImpl out;
            out.write_InputStreamImpl(this, p);
            InputStreamImpl* in = out.create_input_stream();
            in -> _OB_ORBInstance(orbInstance_);
            val = in;
	}
	break;

    case tk_string:
    {
	val = read_string();
	break;
    }
	
    case tk_native:
    case tk_alias: // Alias not possible here

    //
    // These are not supported in OBE
    //
    case tk_fixed:
    case tk_wchar:
    case tk_wstring:
    case tk_value:
    case tk_value_box:
    case tk_abstract_interface:

        assert(false);
	break; // this is to keep the compiler happy
    }

    any.replace(tc, val, true);
}

// ----------------------------------------------------------------------
// OutputStreamImpl public member implementations
// ----------------------------------------------------------------------

void
OutputStreamImpl::write_TypeCode(TypeCode_ptr tc)
{
    //
    // NOTE:
    //
    // No data with natural alignment of greater than four octets is
    // needed for TypeCode. Therefore it is not necessary to do
    // encapsulation in a seperate buffer.
    //

    if(is_nil(tc))
        throw BAD_TYPECODE();

    WriteTCHistory history(11);
    TypeCodeWriter::write(this, tc, history);
}

void
OutputStreamImpl::write_any(const Any& value)
{
    write_TypeCode(value.type_);
    write_any_NoTypeCode(value);
}

void
OutputStreamImpl::write_any_NoTypeCode(const Any& any)
{
    TypeCode_ptr p = GetOrigType_nodup(any.type_);
    
    switch(p -> kind())
    {
    case tk_null:
    case tk_void:
        break;
	    
    case tk_short:
	write_short(*(const Short*)any.value_);
	break;
	
    case tk_long:
	write_long(*(const Long*)any.value_);
	break;

    case tk_longlong:
	write_longlong(*(const LongLong*)any.value_);
	break;
	
    case tk_ushort:
	write_ushort(*(const UShort*)any.value_);
	break;
	
    case tk_ulong:
    case tk_enum:
	write_ulong(*(const ULong*)any.value_);
	break;

    case tk_ulonglong:
	write_ulonglong(*(const ULongLong*)any.value_);
	break;

    case tk_float:
	write_float(*(const Float*)any.value_);
	break;
	
    case tk_double:
	write_double(*(const Double*)any.value_);
	break;
	
    case tk_longdouble:
	write_longdouble(*(const LongDouble*)any.value_);
	break;
	
    case tk_boolean:
	write_boolean(*(const Boolean*)any.value_);
	break;
	
    case tk_char:
	write_char(*(const Char*)any.value_);
	break;

    case tk_octet:
	write_octet(*(const Octet*)any.value_);
	break;
	
    case tk_any:
	write_any(*(const Any*)any.value_);
	break;
	
    case tk_TypeCode:
	write_TypeCode((TypeCode_ptr)any.value_);
	break;
	
    case tk_Principal:
	write_Principal((Principal_ptr)any.value_);
	break;
	
    case tk_objref:
	write_Object((Object_ptr)any.value_);
	break;
	
    case tk_struct:
    case tk_sequence:
    case tk_union:
    case tk_array:
    case tk_except:

	if(p -> _OB_isSystemException())
	{
	    ((SystemException*)any.value_) -> _OB_marshal(this);
	}
	else if(any.info_)
	{
	    any.info_ -> marshal(any.value_, this);
	}
	else
	{
            InputStreamImpl* in = (InputStreamImpl*)any.value_;
            in -> _OB_reset();
	    write_InputStreamImpl(in, p);
	}
	break;
	
    case tk_string:
	write_string((const char*)any.value_);
	break;
	
    case tk_native: // Native not allowed here
    case tk_alias: // Alias not possible here

    //
    // These are not supported in OBE
    //
    case tk_fixed:
    case tk_wchar:
    case tk_wstring:
    case tk_value:
    case tk_value_box:
    case tk_abstract_interface:

        assert(false);
        break; // this is to keep the compiler happy
    }
}

void
OutputStreamImpl::write_InputStreamImpl(InputStreamImpl* in, TypeCode_ptr tc)
{
    switch(tc -> kind())
    {
    case tk_null:
    case tk_void:
	break;
	
    case tk_boolean:
	write_boolean(in -> read_boolean());
	break;

    case tk_char:
	write_char(in -> read_char());
	break;

    case tk_octet:
	write_octet(in -> read_octet());
	break;

    case tk_short:
	write_short(in -> read_short());
	break;

    case tk_ushort:
	write_ushort(in -> read_ushort());
	break;
    
    case tk_long:
	write_long(in -> read_long());
	break;
    
    case tk_ulong:
    case tk_enum:
	write_ulong(in -> read_ulong());
	break;
    
    case tk_float:
	write_float(in -> read_float());
	break;
    
    case tk_longlong:
	write_longlong(in -> read_longlong());
	break;
    
    case tk_ulonglong:
	write_ulonglong(in -> read_ulonglong());
	break;
    
    case tk_double:
	write_double(in -> read_double());
	break;

    case tk_longdouble:
	write_longdouble(in -> read_longdouble());
	break;

    case tk_any:
    {
	// Don't do this:
	//
	// Any_var any = in -> read_any();
	// write_any(any);
	//
	// This is faster:

	TypeCode_var p = in -> read_TypeCode();
	write_TypeCode(p);
	write_InputStreamImpl(in, p);
	break;
    }
    
    case tk_TypeCode:
    {
        // Don't do this:
        //
        // TypeCode_var p = in -> read_TypeCode();
        // write_TypeCode(p);
        //
        // This is faster:

        TCKind kind = (TCKind)in -> read_ulong();

        //
        // An indirection is not permitted at this level
        //
        assert(kind != (TCKind)0xffffffff);

        write_ulong((ULong)kind);

        switch(kind)
        {
        case tk_null:
        case tk_void:
        case tk_short:
        case tk_long:
        case tk_ushort:
        case tk_ulong:
        case tk_float:
        case tk_double:
        case tk_boolean:
        case tk_char:
        case tk_octet:
        case tk_any:
        case tk_TypeCode:
        case tk_Principal:
        case tk_longlong:
        case tk_ulonglong:
        case tk_longdouble:
            break;

        case tk_objref:
        case tk_struct:
        case tk_except:
        case tk_union:
        case tk_enum:
        case tk_sequence:
        case tk_array:
        case tk_alias:
        case tk_native:
        {
            ULong len = in -> read_ulong();
            write_ulong(len);
            addCapacity(len);
            in -> read_octet_array(buf_.cur_, len);
            buf_.cur_ += len;
            break;
        }

        case tk_string:
        {
            ULong bound = in -> read_ulong();
            write_ulong(bound);
            break;
        }

	//
	// These are not supported in OBE
	//
        case tk_fixed:
        case tk_wchar:
        case tk_wstring:
        case tk_value:
        case tk_value_box:
        case tk_abstract_interface:
	    assert(false);
	    break; // this is to keep the compiler happy
        }

        break;
    }
    
    case tk_Principal:
    {
	Principal_var p = in -> read_Principal();
	write_Principal(p);
	break;
    }

    case tk_objref:
    {
	// Don't do this:
	//
	// Object_var p = in -> read_Object();
	// write_Object(p);
	//
	// This is faster:

	IOP::IOR ior;
	IOP::IOR::_OB_unmarshal(ior, in);
	ior._OB_marshal(this);
	break;
    }

    case tk_struct:
    {
	for(ULong i = 0 ; i < tc -> memberTypes_.length() ; i++)
	    write_InputStreamImpl(in, tc -> memberTypes_[i]);
	
	break;
    }
	
    case tk_except:
    {
	String_var id = in -> read_string();
	write_string(id);

	for(ULong i = 0 ; i < tc -> memberTypes_.length() ; i++)
	    write_InputStreamImpl(in, tc -> memberTypes_[i]);
	
	break;
    }
	
    case tk_union:
    {
	Long defaultIndex = tc -> _OB_defaultIndex();
	Long memberIndex = -1;

	TypeCode_ptr discType = GetOrigType_nodup(tc -> discriminatorType_);

	switch(discType -> kind())
	{
	case tk_short:
	{
	    Short val = in -> read_short();
	    write_short(val);
	    
	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(Short*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }
	    
	    break;
	}

	case tk_ushort:
	{
	    UShort val = in -> read_ushort();
	    write_ushort(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(UShort*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_long:
	{
	    Long val = in -> read_long();
	    write_long(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(Long*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_ulong:
	case tk_enum:
	{
	    ULong val = in -> read_ulong();
	    write_ulong(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(ULong*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_longlong:
	{
	    LongLong val = in -> read_longlong();
	    write_longlong(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(LongLong*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_ulonglong:
	{
	    ULongLong val = in -> read_ulonglong();
	    write_ulonglong(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((ULong)i != (ULong)defaultIndex)
		    if(val == *(ULongLong*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_char:
	{
	    Char val = in -> read_char();
	    write_char(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(Char*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_boolean:
	{
	    Boolean val = in -> read_boolean();
	    write_boolean(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(Boolean*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	default:
	    assert(false);
	    break;
	}

	if(memberIndex >= 0)
	    write_InputStreamImpl(in, tc -> memberTypes_[memberIndex]);
	else if(defaultIndex >= 0)
	    write_InputStreamImpl(in, tc -> memberTypes_[defaultIndex]);

	break;
    }
	
    case tk_string:
    {
	String_var s = in -> read_string();
	write_string(s);
	break;
    }

    case tk_sequence:
    case tk_array:
    {
	ULong len;

	if(tc -> kind() == tk_sequence)
	{
	    len = in -> read_ulong();
	    write_ulong(len);
	}
	else
	    len = tc -> length_;
	
	if(len > 0)
	{
	    assert(!is_nil(tc -> contentType_));

	    TypeCode_ptr origContentType = 
		GetOrigType_nodup(tc -> contentType_);
	    
	    switch(origContentType -> kind())
	    {
	    case tk_null:
	    case tk_void:
		break;
		
	    case tk_short:
	    {
                ULong arrsize = len * 2;
                addCapacity(arrsize, 2);
		in -> read_short_array((Short*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }
	    
	    case tk_ushort:
	    {
                ULong arrsize = len * 2;
                addCapacity(arrsize, 2);
		in -> read_ushort_array((UShort*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }
	    
	    case tk_long:
	    {
                ULong arrsize = len * 4;
                addCapacity(arrsize, 4);
		in -> read_long_array((Long*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }

	    case tk_ulong:
	    {
                ULong arrsize = len * 4;
                addCapacity(arrsize, 4);
		in -> read_ulong_array((ULong*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }

	    case tk_float:
	    {
                ULong arrsize = len * 4;
                addCapacity(arrsize, 4);
		in -> read_float_array((Float*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }

	    case tk_longlong:
	    {
                ULong arrsize = len * 8;
                addCapacity(arrsize, 8);
		in -> read_longlong_array((LongLong*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }

	    case tk_ulonglong:
	    {
                ULong arrsize = len * 8;
                addCapacity(arrsize, 8);
		in -> read_ulonglong_array((ULongLong*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }
	    
	    case tk_double:
	    {
                ULong arrsize = len * 8;
                addCapacity(arrsize, 8);
		in -> read_double_array((Double*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }

	    case tk_longdouble:
	    {
                ULong arrsize = len * 16;
                addCapacity(arrsize, 8);
		in -> read_longdouble_array((LongDouble*)(buf_.cur_), len);
		buf_.cur_ += arrsize;
		break;
	    }

	    case tk_boolean:
	    {
                addCapacity(len);
		in -> read_boolean_array((Boolean*)(buf_.cur_), len);
		buf_.cur_ += len;
		break;
	    }

	    case tk_char:
	    {
                addCapacity(len);
		in -> read_char_array((Char*)(buf_.cur_), len);
		buf_.cur_ += len;
		break;
	    }

	    case tk_octet:
	    {
                addCapacity(len);
		in -> read_octet_array(buf_.cur_, len);
		buf_.cur_ += len;
		break;
	    }

	    //
	    // These are not supported in OBE
	    //
	    case tk_wchar:

	    case tk_alias:
		assert(false);
		break;

	    default:
		for(ULong i = 0 ; i < len ; i++)
		    write_InputStreamImpl(in, tc -> contentType_);
		break;
	    }
	}

	break;
    }

    case tk_alias:
	write_InputStreamImpl(in, tc -> contentType_);
	break;

    //
    // These are not supported in OBE
    //
    case tk_fixed:
    case tk_wchar:
    case tk_wstring:
    case tk_value:
    case tk_value_box:
    case tk_abstract_interface:

    case tk_native:
        assert(false);
	break; // this is to keep the compiler happy
    }
}
