// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/Except.h>
#include <OBE/Upcall.h>
#include <OBE/ORBInstance.h>
#include <OBE/Stream.h>
#include <OBE/RefCounted.h>

#include <OBE/Object.h>

#include <OBE/POAServantBase.h>
#include <OBE/POA.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// UpcallImpl constructor and destructor
// ----------------------------------------------------------------------

//
// Note that we do not duplicate ORBInstance. We know ORBInstance
// will exist until the ORB is destroyed so it will not go away 
// underneath us. 
//
UpcallImpl::UpcallImpl(ORBInstance_ptr orbInstance)
    : orbInstance_(orbInstance),
      upcallReturn_(0),
      reqId_(0),
      op_(0),
      out_(12),
      in_(12, 0, false),
      poa_(0)
{
    in_._OB_ORBInstance(orbInstance_);
}

UpcallImpl::~UpcallImpl()
{
}

// ----------------------------------------------------------------------
// Upcall public member implementation
// ----------------------------------------------------------------------

void
UpcallImpl::setup(UpcallReturn* upcallReturn,
		  const OCI::ObjectKey* ocikey,
		  ULong requestId,
		  const char* op)
{
    upcallReturn_ = upcallReturn;
    ocikey_ = ocikey;
    reqId_ = requestId;
    op_ = op;
}

bool
UpcallImpl::responseExpected() const
{
    return (upcallReturn_ != 0);
}

InputStreamImpl*
UpcallImpl::preUnmarshal()
    throw(SystemException, LocationForward)
{
    return &in_;
}

void
UpcallImpl::unmarshalEx(const SystemException& ex)
    throw(SystemException, LocationForward)
{
    ex._raise();
}

void
UpcallImpl::postUnmarshal()
    throw(SystemException, LocationForward)
{
}

OutputStreamImpl*
UpcallImpl::preMarshal()
    throw(SystemException, LocationForward)
{
    if(upcallReturn_ != 0)
	upcallReturn_ -> upcallBeginReply(this);
    return &out_;
}

void
UpcallImpl::marshalEx(const SystemException& ex)
    throw(SystemException, LocationForward)
{
    ex._raise();
}

void
UpcallImpl::postMarshal()
    throw(SystemException, LocationForward)
{
    if(upcallReturn_ != 0)
        upcallReturn_ -> upcallEndReply(this);
}

void
UpcallImpl::setUserException(const UserException& ex)
    throw()
{
    if(upcallReturn_ != 0)
        upcallReturn_ -> upcallUserException(this, ex);
}

void
UpcallImpl::setSystemException(const SystemException& ex)
    throw()
{
    if(upcallReturn_ != 0)
        upcallReturn_ -> upcallSystemException(this, ex);
}

void
UpcallImpl::setLocationForward(RefCountIOR_ptr ior)
    throw()
{
    if(upcallReturn_ != 0)
        upcallReturn_ -> upcallForward(this, ior);
}

//
// Note that we do not duplicate POA. We know the POA will exist until
// the ORB is destroyed so it will not go away underneath us and the 
// pointer will remain valid. 
//
void
UpcallImpl::setDispatchInfo(PortableServer::POA_ptr poa,
			    const ObjectKeyData& key)
    throw()
{
    assert(!is_nil(poa));
    poa_ = poa;
    key_ = key;
}

void
UpcallImpl::invoke()
    throw()
{
    try
    {
	assert(!is_nil(poa_));
	poa_ -> _OB_dispatch(key_, this);
    }
    catch(const SystemException& ex)
    {
	setSystemException(ex);
    }
}
