// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <TestIntf_2_3.h>
#include <TestMacro.h>

#include <errno.h>

using namespace CORBA;

int
ClientRun(ORB_ptr orb, bool local, int argc, char* argv[])
{
    int i;

    //
    // Get TestIntf
    //
    const char* refFile = "TestIntf.ref";
    FILE* f = fopen(refFile, "r");
    if(f == 0)
    {
	fprintf(stderr, "%s: can't open `%s`: %s\n",
		argv[0], refFile, strerror(errno));
	return EXIT_FAILURE;
    }

    //
    // Get the stringified IORs
    //
    String_var impl[2];
    char buf[1024];
    for(i = 0 ; i < 2 ; i++)
    {
	//
	// Read the IOR from the file
	//
	if(fgets(buf, 1024, f) == 0)
	{
	    fprintf(stderr, "%s: can't read IOR %d from `%s`: %s\n",
		    argv[0], i + 1, refFile, strerror(errno));
	    return EXIT_FAILURE;
	}

	//
	// Null terminate the IOR
	//
	char* p = strpbrk(buf, "\r\n");
	if(p != 0)
	    *p = '\0';

	//
	// Save the IOR
	//
	impl[i] = string_dup(buf);
    }

    fclose(f); // The explicit close is necessary to satisfy the Windows NT
               // file locking

    //
    // Run tests
    //

    printf("Testing string_to_object()... ");
    fflush(stdout);
    Object_var obj[2];
    for(i = 0 ; i < 2 ; i++)
    {
	obj[i] = orb -> string_to_object(impl[i]);
	TEST(!is_nil(obj[i]));
    }
    printf("OK!\n");

    printf("Testing _narrow()... ");
    fflush(stdout);
    TestIntf_var ti[2];
    for(i = 0 ; i < 2 ; i++)
    {
	ti[i] = TestIntf::_narrow(obj[i]);
	TEST(!is_nil(ti[i]));
    }
    TestIntf_2_3_NoLongDouble_var ti_2_3_nld[2];
    for(i = 0 ; i < 2 ; i++)
	ti_2_3_nld[i] = TestIntf_2_3_NoLongDouble::_narrow(obj[i]);
    TestIntf_2_3_var ti_2_3[2];
    for(i = 0 ; i < 2 ; i++)
	ti_2_3[i] = TestIntf_2_3::_narrow(obj[i]);
    printf("OK!\n");

    const char* objDesc[] = 
    {
	"SSI",
	"Tie"
    };

    extern void TestStub(TestIntf_ptr);
    printf("Testing the SII w/ 2.0 types:\n");
    for(i = 0 ; i < 2 ; i++)
    {
	printf("\tWith the %s implementation... ", objDesc[i]);
        fflush(stdout);
	TestStub(ti[i]);
	printf("OK!\n");
    }

    if(!is_nil(ti_2_3_nld[0]))
    {
	extern void TestStub_2_3_NoLongDouble(TestIntf_2_3_NoLongDouble_ptr);
	printf("Testing the SII w/ 2.3 types w/o long double type:\n");
	for(i = 0 ; i < 2 ; i++)
	{
	    printf("\tWith the %s implementation... ", objDesc[i]);
            fflush(stdout);
	    TestStub_2_3_NoLongDouble(ti_2_3_nld[i]);
	    printf("OK!\n");
	}
    }

    if(!is_nil(ti_2_3[0]))
    {
	extern void TestStub_2_3(TestIntf_2_3_ptr);
	printf("Testing the SII w/ long double type:\n");
	for(i = 0 ; i < 2 ; i++)
	{
	    printf("\tWith the %s implementation... ", objDesc[i]);
            fflush(stdout);
	    TestStub_2_3(ti_2_3[i]);
	    printf("OK!\n");
	}
    }

    extern void TestStubException(TestIntf_ptr, bool);
    printf("Testing the SII and exceptions w/ 2.0 types:\n");
    for(i = 0 ; i < 2 ; i++)
    {
	printf("\tWith the %s implementation... ", objDesc[i]);
        fflush(stdout);
	TestStubException(ti[i], !local);
	printf("OK!\n");
    }

    if(!is_nil(ti_2_3_nld[0]))
    {
	extern void TestStubException_2_3_NoLongDouble(
	    TestIntf_2_3_NoLongDouble_ptr, bool);
	printf("Testing SII and exceptions w/ 2.3 types "
	       "w/o long double type:\n");
	for(i = 0 ; i < 2 ; i++)
	{
	    printf("\tWith the %s implementation... ", objDesc[i]);
            fflush(stdout);
	    TestStubException_2_3_NoLongDouble(ti_2_3_nld[i], !local);
	    printf("OK!\n");
	}
    }

    if(!is_nil(ti_2_3[0]))
    {
	extern void TestStubException_2_3(TestIntf_2_3_ptr, bool);
	printf("Testing SII and exceptions w/ long double type:\n");
	for(i = 0 ; i < 2 ; i++)
	{
	    printf("\tWith the %s implementation... ", objDesc[i]);
            fflush(stdout);
	    TestStubException_2_3(ti_2_3[i], !local);
	    printf("OK!\n");
	}
    }

    if(!local)
    {
	extern void TestStubTimeout(TestIntf_ptr, ORB_ptr);
        printf("Testing the SII w/ timeouts:\n");
	for(i = 0 ; i < 2 ; i++)
	{
	    printf("\tWith the %s implementation... ", objDesc[i]);
            fflush(stdout);
	    TestStubTimeout(ti[i], orb);
	    printf("OK!\n");
	}
    }

    //
    // Deactivate the server
    //
    ULong version = ti[0] -> version();
    if(version >= 3999900L)
    {
	ti[0] -> deactivate();
    }
    else
    {
	try
	{
	    ti[0] -> deactivate();
	}
	catch(const TRANSIENT&)
	{
	}
    }

    return EXIT_SUCCESS;
}
