// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <TestIntf_2_3_skel_tie.h>
#include <Test_impl.h>
#include <TestMacro.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

#include <errno.h>

using namespace CORBA;

//
// Ref counted tie class
//
class TestIntf_2_3_refcount_tie
    : public POA_TestIntf_2_3_tie< TestIntf_2_3_impl >,
      public PortableServer::RefCountServantBase
{
    PortableServer::ServantBase_var servant;

public:

    TestIntf_2_3_refcount_tie(TestIntf_2_3_impl* impl,
			      PortableServer::POA_ptr poa)
	: POA_TestIntf_2_3_tie< TestIntf_2_3_impl >(impl, poa, false)
    {
	impl -> _add_ref();
        servant = impl;
    }
};

int
ServerRun(ORB_ptr orb, bool nonBlocking, int argc, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poaObj);
	    
    //
    // Activate the POA manager
    //
    PortableServer::POAManager_var manager = poa -> the_POAManager();
    manager -> activate();
    
    //
    // Create implementation objects
    //
    TestIntf_2_3_impl* i = new TestIntf_2_3_impl(orb);
    PortableServer::ServantBase_var servant = i;
    TestIntf_2_3_var p = i -> _this();
    String_var impl = orb -> object_to_string(p);
    
    TestIntf_2_3_refcount_tie* iTie = new TestIntf_2_3_refcount_tie(i, poa);
    PortableServer::ServantBase_var servantTie = iTie;
    Object_var pTie = iTie -> _this();
    String_var implTie = orb -> object_to_string(pTie);

    //
    // Save references. This must be done after POA manager activation,
    // otherwise there is a potential for a race condition between the
    // client sending request and the server not being ready yet.
    //
    FILE* f = fopen("TestIntf.ref", "w");
    if(f == 0)
    {
	fprintf(stderr, "%s: can't open `TestIntf.ref`: %s\n",
		argv[0], strerror(errno));
	return EXIT_FAILURE;
    }
    
    fprintf(f, "%s\n", impl.in());
    fprintf(f, "%s\n", implTie.in());

    fclose(f);

    if(!nonBlocking)
    {
	//
	// Give up control to the ORB
	//
        orb -> run();

        //
        // Clean up
        //
	void ServerCleanup();
	ServerCleanup();
    }

    return EXIT_SUCCESS;
}

void
ServerCleanup()
{
    //
    // Remove reference file
    //
#ifdef HAVE__UNLINK
    _unlink("TestIntf.ref");
#else
    unlink("TestIntf.ref");
#endif
}
