// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <TestIntf_2_3.h>
#include <TestMacro.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// Tests for TestIntf
// ----------------------------------------------------------------------

void
TestStubException(TestIntf_ptr ti, bool remote)
{
    ULong i, j, k, l;

    ULong version = ti -> version();

    {
	try
	{
	    ti -> opVoidEx();
	    TEST(false);
	}
	catch(TestIntf::ExVoid&)
	{
	}
    }

    {
	Short inOut, out;
	inOut = 20;

	try
	{
	    ti -> opShortEx(10, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExShort& ex)
	{
	    TEST(ex.value == 30);
	}
    }

    {
	Long inOut, out;
	inOut = 20;

	try
	{
	    ti -> opLongEx(10, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExLong& ex)
	{
	    TEST(ex.value == 30);
	}
    }

    {
	UShort inOut, out;
	inOut = 20;

	try
	{
	    ti -> opUShortEx(10, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExUShort& ex)
	{
	    TEST(ex.value == 30);
	}
    }

    {
	ULong inOut, out;
	inOut = 20;

	try
	{
	    ti -> opULongEx(10, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExULong& ex)
	{
	    TEST(ex.value == 30);
	}
    }

    {
	Float inOut, out;
	inOut = 20;

	try
	{
	    ti -> opFloatEx(10, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExFloat& ex)
	{
	    TEST(ex.value == 30);
	}
    }

    {
	Double inOut, out;
	inOut = 20;

	try
	{
	    ti -> opDoubleEx(10, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExDouble& ex)
	{
	    TEST(ex.value == 30);
	}
    }

    {
	Boolean inOut, out;
	inOut = true;

	try
	{
	    ti -> opBooleanEx(true, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExBoolean& ex)
	{
	    TEST(ex.value == true);
	}
    }

    {
	Char inOut, out;
	inOut = 1;

	try
	{
	    ti -> opCharEx('a', inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExChar& ex)
	{
	    TEST(ex.value == 'b');
	}
    }

    {
	Octet inOut, out;
	inOut = 20;

	try
	{
	    ti -> opOctetEx(10, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExOctet& ex)
	{
	    TEST(ex.value == 30);
	}
    }

    {
	String_var inOut, out;
	inOut = string_dup("world!");

	try
	{
	    ti -> opStringEx("Hello, ", inOut.inout(), out.out());
	    TEST(false);
	}
	catch(TestIntf::ExString& ex)
	{
	    TEST(strcmp(ex.value, "Hello, world!") == 0);
	}
    }

    {
	TestIntf::VariableStruct vStruct;
	vStruct.s = string_dup("xyz");
	Any any;
	any <<= vStruct;
	Any inOut;
	Any_var out;

	try
	{
	    ti -> opAnyEx(any, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExAny& ex)
	{
	    const TestIntf::VariableStruct* vStructRet;
	    TEST(ex.value >>= vStructRet);
	    TEST(strcmp(vStructRet -> s, "xyz") == 0);
	}
    }

    {
	TestIntf::TestEnum inOut, out;
	inOut = TestIntf::TestEnum2;

	try
	{
	    ti -> opTestEnumEx(TestIntf::TestEnum3, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExTestEnum& ex)
	{
	    TEST(ex.value == TestIntf::TestEnum3);
	}
    }

    if(version >= 3999900L)
    {
	TestIntf_var inOut, out;
	inOut = TestIntf::_duplicate(ti);
	try
	{
	    ti -> opTestIntfEx(ti, inOut.inout(), out.out());
	    TEST(false);
	}
	catch(TestIntf::ExTestIntf& ex)
	{
    /* XXX
	    TEST(ex.value -> _hash(1000) == ti -> _hash(1000));
	    TEST(ex.value -> _is_equivalent(ti));
    */
	}
    }

    {
	TestIntf::FixedStruct st;
	st.s = 100;
	st.l = -100;
	TestIntf::FixedStruct inOut, out;
	inOut.s = 10000;
	inOut.l = 100000;
	    
	try
	{
	    ti -> opFixedStructEx(st, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExFixedStruct& ex)
	{
	    TEST(ex.value.s == st.s);
	    TEST(ex.value.l == st.l);
	}

	{
	    TestIntf::VariableStruct st;
	    st.s = string_dup("$$$");
	    TestIntf::VariableStruct inOut;
	    TestIntf::VariableStruct_var out;
	    inOut.s = string_dup("bla");

	    try
	    {
		ti -> opVariableStructEx(st, inOut, out);
		TEST(false);
	    }
	    catch(TestIntf::ExVariableStruct& ex)
	    {
		TEST(strcmp(ex.value.s, st.s) == 0);
	    }
	}

	{
	    TestIntf::FixedUnion un;
	    un.b(true);
	    un._d(999);
	    TestIntf::FixedUnion inOut, out;
	    inOut.l(100);

	    try
	    {
		ti -> opFixedUnionEx(un, inOut, out);
		TEST(false);
	    }
	    catch(TestIntf::ExFixedUnion& ex)
	    {
		TEST(ex.value._d() == 999);
		TEST(ex.value.b() == true);
	    }
	}

	if(version >= 3999900L)
	{
	    TestIntf::VariableUnion un;
	    un.ti(ti);
	    TestIntf::VariableUnion inOut;
	    TestIntf::VariableUnion_var out;
	    TestIntf::VariableStruct st;
	    st.s = string_dup("bla");
	    inOut.st(st);

	    try
	    {
		ti -> opVariableUnionEx(un, inOut, out);
		TEST(false);
	    }
	    catch(TestIntf::ExVariableUnion& ex)
	    {
    /* XXX
		TEST(ex.value.ti() -> _hash(2000) == ti -> _hash(2000));
		TEST(ex.value.ti() -> _is_equivalent(ti));
    */
	    }
	}
    }

    {
	TestIntf::StringSequence seq;
	seq.length(3);
	seq[0] = string_dup("!!!");
	seq[1] = string_dup("@@@");
	seq[2] = string_dup("###");
	TestIntf::StringSequence inOut;
	TestIntf::StringSequence_var out;
	inOut.length(2);
	inOut[0] = string_dup("%");
	inOut[1] = string_dup("^^");

	try
	{
	    ti -> opStringSequenceEx(seq, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExStringSequence& ex)
	{
	    TEST(ex.value.length() == 5);
	    TEST(strcmp(ex.value[0], "!!!") == 0);
	    TEST(strcmp(ex.value[1], "@@@") == 0);
	    TEST(strcmp(ex.value[2], "###") == 0);
	    TEST(strcmp(ex.value[3], "%") == 0);
	    TEST(strcmp(ex.value[4], "^^") == 0);
	}
    }

//
// For some reason this test fails with gcc 2.95.2 with optimization
// and static libraries :-(
//
#if !defined(__GNUC__) || ! defined(__OPTIMIZE__) // COMPILERFIX
    {
	TestIntf::FixedArray ar =
	{
	    {
		{ 1, 2, 3, 4 },
		{ 10, -10, 11, -11 },
		{ -999, 0, 888, 123 }
	    },
	    {
		{ 17, 27, 37, 47 },
		{ 710, -710, 711, -711 },
		{ -99, 0, 88, 13 }
	    }
	};
	
	TestIntf::FixedArray_var ret;
	TestIntf::FixedArray inOut, out;
	for(i = 0 ; i < 2 ; i++)
	    for(j = 0 ; j < 3 ; j++)
		for(k = 0 ; k < 4 ; k++)
		    inOut[i][j][k] = (short)(i + j + k);

	try
	{
#ifdef HAVE_VCPLUSPLUS_BUGS
	    ti -> opFixedArrayEx((const TestIntf::FixedArray_slice*)ar,
				 inOut, out);
#else
	    ti -> opFixedArrayEx(ar, inOut, out);
#endif
	    TEST(false);
	}
	catch(TestIntf::ExFixedArray& ex)
	{
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
		    {
			TEST(ar[i][j][k] == ex.value[i][j][k]);
		    }
	}
    }
#endif

//
// For some reason the following test fails with SGI CC 7.2.1 and
// DEC C++ 6.2-024 :-( I have no idea why.
//
#if !defined(__sgi) && !defined(__DECCXX)
    {
	TestIntf::VariableArray ar =
	{ 
	    {
		string_dup("aa"),
		string_dup("bb"),
		string_dup("cc")
	    },
	    {
		string_dup("AA"),
		string_dup("BB"),
		string_dup("CC")
	    }
	};
	
	TestIntf::VariableArray_var ret;
	TestIntf::VariableArray inOut;
	for(i = 0 ; i < 2 ; i++)
	    for(j = 0 ; j < 3 ; j++)
		inOut[i][j] = string_dup("abc");
	TestIntf::VariableArray_var out;

	try
	{
#ifdef HAVE_VCPLUSPLUS_BUGS
	    ti -> opVariableArrayEx((const TestIntf::VariableArray_slice*)
				    ar, inOut, out.out());
#else
	    ti -> opVariableArrayEx(ar, inOut, out.out());
#endif
	    TEST(false);
	}
	catch(TestIntf::ExVariableArray& ex)
	{
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		{
		    TEST(strcmp(ar[i][j], ex.value[i][j]) == 0);
		}
	}
    }
#endif

    {
	TestIntf::FixedArray ar0 =
	{
	    {
		{ 1, 2, 3, 4 },
		{ 10, -10, 11, -11 },
		{ -999, 0, 888, 123 }
	    },
	    {
		{ 17, 27, 37, 47 },
		{ 710, -710, 711, -711 },
		{ -99, 0, 88, 13 }
	    }
	};
	
	TestIntf::FixedArray ar1 =
	{
	    {
		{ 2, 3, 4, 1 },
		{ 10, 11, 11, -10 },
		{ -0, 239, 123, 888 }
	    },
	    {
		{ 17, 37, 47, 27 },
		{ 710, -710, 711, -711 },
		{ -0, -99, 13, 8338 }
	    }
	};
	
	TestIntf::FixedArray ar2 =
	{
	    {
		{ 1, 2, -3, -234 },
		{ 10, -11, 11, -10 },
		{ -999, 30, 1888, 123 }
	    },
	    {
		{ 27, 37, 117, 47 },
		{ 710, -7150, 711, -711 },
		{ -0, 13, 929, 88 }
	    }
	};
	
	TestIntf::FixedArraySequence seq;
	seq.length(3);
	TestIntf::FixedArray_copy(seq[0], ar0);
	TestIntf::FixedArray_copy(seq[1], ar1);
	TestIntf::FixedArray_copy(seq[2], ar2);

	TestIntf::FixedArraySequence inOut;
	inOut.length(4);
	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
			inOut[l][i][j][k] = (short)(i + j + k + l);

	TestIntf::FixedArraySequence_var ret;
	TestIntf::FixedArraySequence_var out;

	try
	{
	    ret = ti -> opFixedArraySequenceEx(seq, inOut, out.out());
	    TEST(false);
	}
	catch(TestIntf::ExFixedArraySequence& ex)
	{
	    TEST(ex.value.length() == 7);
	    
	    for(l = 0 ; l < 3 ; l++)
		for(i = 0 ; i < 2 ; i++)
		    for(j = 0 ; j < 3 ; j++)
			for(k = 0 ; k < 4 ; k++)
			{
			    TEST(seq[l][i][j][k] == ex.value[l][i][j][k]);
			}

	    for(l = 0 ; l < 4 ; l++)
		for(i = 0 ; i < 2 ; i++)
		    for(j = 0 ; j < 3 ; j++)
			for(k = 0 ; k < 4 ; k++)
			{
			    TEST(ex.value[3 + l][i][j][k] ==
				 (Long)(i + j + k + l));
			}
	}
    }

    {
	TestIntf::VariableArray ar0 =
	{ 
	    {
		string_dup("aa"),
		string_dup("bb"),
		string_dup("cc")
	    },
	    {
		string_dup("AA"),
		string_dup("BB"),
		string_dup("CC")
	    }
	};

	TestIntf::VariableArray ar1 =
	{ 
	    {
		string_dup("a~a"),
		string_dup("b~b"),
		string_dup("c~c")
	    },
	    {
		string_dup("A~A"),
		string_dup("B~B"),
		string_dup("C~C")
	    }
	};

	TestIntf::VariableArraySequence seq;
	seq.length(2);
	TestIntf::VariableArray_copy(seq[0], ar0);
	TestIntf::VariableArray_copy(seq[1], ar1);

	TestIntf::VariableArraySequence inOut;
	inOut.length(4);
	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    inOut[l][i][j] = string_dup("***");

	TestIntf::VariableArraySequence_var ret;
	TestIntf::VariableArraySequence_var out;
	
	try
	{
	    ret = ti -> opVariableArraySequenceEx(seq, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExVariableArraySequence& ex)
	{
	    TEST(ex.value.length() == 6);
	    
	    for(l = 0 ; l < 2 ; l++)
		for(i = 0 ; i < 2 ; i++)
		    for(j = 0 ; j < 3 ; j++)
		    {
			TEST(strcmp(seq[l][i][j],
				    ex.value[l][i][j]) == 0);
		    }
	    
	    for(l = 0 ; l < 4 ; l++)
		for(i = 0 ; i < 2 ; i++)
		    for(j = 0 ; j < 3 ; j++)
		    {
			TEST(strcmp(ex.value[2 + l][i][j], "***") == 0);
		    }
	}
    }

    {
	TestIntf::FixedArray ar0 =
	{
	    {
		{ 1, 2, 3, 4 },
		{ 10, -10, 11, -11 },
		{ -999, 0, 888, 123 }
	    },
	    {
		{ 17, 27, 37, 47 },
		{ 710, -710, 711, -711 },
		{ -99, 0, 88, 13 }
	    }
	};
	
	TestIntf::FixedArray ar1 =
	{
	    {
		{ 2, 3, 4, 1 },
		{ 10, 11, 11, -10 },
		{ -0, 239, 123, 888 }
	    },
	    {
		{ 17, 37, 47, 27 },
		{ 710, -710, 711, -711 },
		{ -0, -99, 13, 8338 }
	    }
	};
	
	TestIntf::FixedArray ar2 =
	{
	    {
		{ 1, 2, -3, -234 },
		{ 10, -11, 11, -10 },
		{ -999, 30, 1888, 123 }
	    },
	    {
		{ 27, 37, 117, 47 },
		{ 710, -7150, 711, -711 },
		{ -0, 13, 929, 88 }
	    }
	};
	
	TestIntf::FixedArrayBoundSequence seq;
	seq.length(3);
	TestIntf::FixedArray_copy(seq[0], ar0);
	TestIntf::FixedArray_copy(seq[1], ar1);
	TestIntf::FixedArray_copy(seq[2], ar2);

	TestIntf::FixedArrayBoundSequence inOut;
	inOut.length(4);
	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    for(k = 0 ; k < 4 ; k++)
			inOut[l][i][j][k] = (short)(i + j + k + l);

	TestIntf::FixedArrayBoundSequence_var ret;
	TestIntf::FixedArrayBoundSequence_var out;

	try
	{
	    ret = ti -> opFixedArrayBoundSequenceEx(seq, inOut, out.out());
	    TEST(false);
	}
	catch(TestIntf::ExFixedArrayBoundSequence& ex)
	{
	    TEST(ex.value.length() == 7);
	    
	    for(l = 0 ; l < 3 ; l++)
		for(i = 0 ; i < 2 ; i++)
		    for(j = 0 ; j < 3 ; j++)
			for(k = 0 ; k < 4 ; k++)
			{
			    TEST(seq[l][i][j][k] == ex.value[l][i][j][k]);
			}

	    for(l = 0 ; l < 4 ; l++)
		for(i = 0 ; i < 2 ; i++)
		    for(j = 0 ; j < 3 ; j++)
			for(k = 0 ; k < 4 ; k++)
			{
			    TEST(ex.value[3 + l][i][j][k] ==
				 (Long)(i + j + k + l));
			}
	}
    }

    {
	TestIntf::VariableArray ar0 =
	{ 
	    {
		string_dup("aa"),
		string_dup("bb"),
		string_dup("cc")
	    },
	    {
		string_dup("AA"),
		string_dup("BB"),
		string_dup("CC")
	    }
	};

	TestIntf::VariableArray ar1 =
	{ 
	    {
		string_dup("a~a"),
		string_dup("b~b"),
		string_dup("c~c")
	    },
	    {
		string_dup("A~A"),
		string_dup("B~B"),
		string_dup("C~C")
	    }
	};

	TestIntf::VariableArrayBoundSequence seq;
	seq.length(2);
	TestIntf::VariableArray_copy(seq[0], ar0);
	TestIntf::VariableArray_copy(seq[1], ar1);

	TestIntf::VariableArrayBoundSequence inOut;
	inOut.length(4);
	for(l = 0 ; l < 4 ; l++)
	    for(i = 0 ; i < 2 ; i++)
		for(j = 0 ; j < 3 ; j++)
		    inOut[l][i][j] = string_dup("***");

	TestIntf::VariableArrayBoundSequence_var ret;
	TestIntf::VariableArrayBoundSequence_var out;
	
	try
	{
	    ret = ti -> opVariableArrayBoundSequenceEx(seq, inOut, out);
	    TEST(false);
	}
	catch(TestIntf::ExVariableArrayBoundSequence& ex)
	{
	    TEST(ex.value.length() == 6);
	    
	    for(l = 0 ; l < 2 ; l++)
		for(i = 0 ; i < 2 ; i++)
		    for(j = 0 ; j < 3 ; j++)
		    {
			TEST(strcmp(seq[l][i][j],
				    ex.value[l][i][j]) == 0);
		    }
	    
	    for(l = 0 ; l < 4 ; l++)
		for(i = 0 ; i < 2 ; i++)
		    for(j = 0 ; j < 3 ; j++)
		    {
			TEST(strcmp(ex.value[2 + l][i][j], "***") == 0);
		    }
	}
    }

    {
	try
	{
	    ti -> op_UNKNOWN_Ex();
	    TEST(false);
	}
	catch(const UNKNOWN& ex)
	{
	    TEST(ex.minor() == 1);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_YES);
	}

	try
	{
	    ti -> op_BAD_PARAM_Ex();
	    TEST(false);
	}
	catch(const BAD_PARAM& ex)
	{
	    TEST(ex.minor() == 2);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_NO);
	}

	try
	{
	    ti -> op_NO_MEMORY_Ex();
	    TEST(false);
	}
	catch(const NO_MEMORY& ex)
	{
	    TEST(ex.minor() == 3);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_MAYBE);
	}

	try
	{
	    ti -> op_IMP_LIMIT_Ex();
	    TEST(false);
	}
	catch(const IMP_LIMIT& ex)
	{
	    TEST(ex.minor() == 4);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_YES);
	}

	try
	{
	    ti -> op_COMM_FAILURE_Ex();
	    TEST(false);
	}
	catch(const COMM_FAILURE& ex)
	{
	    TEST(ex.minor() == 5);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_NO);
	}

	try
	{
	    ti -> op_INV_OBJREF_Ex();
	    TEST(false);
	}
	catch(const INV_OBJREF& ex)
	{
	    TEST(ex.minor() == 6);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_MAYBE);
	}

	try
	{
	    ti -> op_NO_PERMISSION_Ex();
	    TEST(false);
	}
	catch(const NO_PERMISSION& ex)
	{
	    TEST(ex.minor() == 7);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_YES);
	}

	try
	{
	    ti -> op_INTERNAL_Ex();
	    TEST(false);
	}
	catch(const INTERNAL& ex)
	{
	    TEST(ex.minor() == 8);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_NO);
	}

	try
	{
	    ti -> op_MARSHAL_Ex();
	    TEST(false);
	}
	catch(const MARSHAL& ex)
	{
	    TEST(ex.minor() == 9);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_MAYBE);
	}

	try
	{
	    ti -> op_INITIALIZE_Ex();
	    TEST(false);
	}
	catch(const INITIALIZE& ex)
	{
	    TEST(ex.minor() == 10);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_YES);
	}

	try
	{
	    ti -> op_NO_IMPLEMENT_Ex();
	    TEST(false);
	}
	catch(const NO_IMPLEMENT& ex)
	{
	    TEST(ex.minor() == 11);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_NO);
	}

	try
	{
	    ti -> op_BAD_TYPECODE_Ex();
	    TEST(false);
	}
	catch(const BAD_TYPECODE& ex)
	{
	    TEST(ex.minor() == 12);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_MAYBE);
	}

	try
	{
	    ti -> op_BAD_OPERATION_Ex();
	    TEST(false);
	}
	catch(const BAD_OPERATION& ex)
	{
	    TEST(ex.minor() == 13);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_YES);
	}

	try
	{
	    ti -> op_NO_RESOURCES_Ex();
	    TEST(false);
	}
	catch(const NO_RESOURCES& ex)
	{
	    TEST(ex.minor() == 14);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_NO);
	}

	try
	{
	    ti -> op_NO_RESPONSE_Ex();
	    TEST(false);
	}
	catch(const NO_RESPONSE& ex)
	{
	    TEST(ex.minor() == 15);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_MAYBE);
	}

	try
	{
	    ti -> op_BAD_INV_ORDER_Ex();
	    TEST(false);
	}
	catch(const BAD_INV_ORDER& ex)
	{
	    TEST(ex.minor() == 17);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_NO);
	}

	try
	{
	    ti -> op_TRANSIENT_Ex();
	    TEST(false);
	}
	catch(const TRANSIENT& ex)
	{
	    TEST(ex.minor() == 18);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_MAYBE);
	}

	try
	{
	    ti -> op_OBJ_ADAPTER_Ex();
	    TEST(false);
	}
	catch(const OBJ_ADAPTER& ex)
	{
	    TEST(ex.minor() == 24);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_MAYBE);
	}

	try
	{
	    ti -> op_OBJECT_NOT_EXIST_Ex();
	    TEST(false);
	}
	catch(const OBJECT_NOT_EXIST& ex)
	{
	    TEST(ex.minor() == 26);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_NO);
	}

	try
	{
	    ti -> op_INV_POLICY_Ex();
	    TEST(false);
	}
	catch(const INV_POLICY& ex)
	{
	    TEST(ex.minor() == 30);
	    //XXX TEST(ex.remote() == remote);
	    TEST(ex.completed() == COMPLETED_NO);
	}
    }
}

// ----------------------------------------------------------------------
// Tests for TestIntf_2_3_NoLongDouble
// ----------------------------------------------------------------------

void
TestStubException_2_3_NoLongDouble(TestIntf_2_3_NoLongDouble_ptr ti,
				   bool remote)
{
}

// ----------------------------------------------------------------------
// Tests for TestIntf_2_3
// ----------------------------------------------------------------------

void
TestStubException_2_3(TestIntf_2_3_ptr ti, bool remote)
{
#ifndef WIN32
#if SIZEOF_LONG_DOUBLE >= 12
    {
	LongDouble inOut, out;
	inOut = 20;

	try
	{
	    ti -> opLongDoubleEx(10, inOut, out);
	    TEST(false);
	}
	catch(TestIntf_2_3::ExLongDouble& ex)
	{
	    TEST(ex.value == 30);
	}
	catch(const MARSHAL& ex)
	{
	    //
	    // Ignore exceptions caused by unsupported data types
	    //
	    if(ex.minor() != OB::MinorLongDoubleNotSupported)
		throw;
	}
    }
#endif
#endif
}
