// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <TestMacro.h>
#include <Test.h>

using namespace CORBA;

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);
        
        Object_var obj = orb -> string_to_object("relfile:/Test.ref");
        if(CORBA::is_nil(obj))
        {
            fprintf(stderr, "%s: cannot read IOR from Test.ref\n", argv[0]);
            return EXIT_FAILURE;
        }
        TestServer_var server = TestServer::_narrow(obj);
        TEST(!is_nil(server));

        TestInfoSeq_var info = server -> get_info();

        for(ULong i = 0 ; i < info -> length() ; i++)
        {
            try
            {
                info[i].obj -> aMethod();
                TEST(strlen(info[i].except_id) == 0);
            }
            catch(const SystemException& ex)
            {
                if(strcmp(info[i].except_id, ex._rep_id()) != 0)
                {
                    fprintf(stderr, "TestClient: Invocation on object #%lu "
			    "raised an unexpected exception\n", i);
                    fprintf(stderr, "Expected %s but caught %s\n",
			    info[i].except_id.in(), ex._rep_id());
                    TEST(false);
                }
            }
        }

	server -> deactivate();
    }
    catch(const Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
        status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
