// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <TestMacro.h>
#include <Test_skel.h>
#include <TestRef_impl.h>
#include <TestUtil.h>

#include <errno.h>

using namespace CORBA;
using namespace PortableServer;

void
TestRefCount(ORB_ptr orb, POA_ptr root)
{
    //
    // Create servant
    //
    TestRef_impl* impl = new TestRef_impl();
    ServantBase_var servant = impl;

    //
    // Activate servant
    //
    ObjectId_var id1 = root -> activate_object(servant);

    {
	//
	// Get servant using reference_to_servant()
	//
	Object_var obj = root -> _OB_createReference(id1, "IDL:Test:1.0");
        ServantBase_var s = root -> reference_to_servant(obj);
    }

    {
	//
	// Get servant using id_to_servant()
	//
        ServantBase_var s = root -> id_to_servant(id1);
    }

    //
    // Activate the servant again
    //
    ObjectId_var id2 = root -> activate_object(servant);

    //
    // Deactivate each object
    //
    root -> deactivate_object(id2);
    root -> deactivate_object(id1);

    //
    // The servant's ref count now should be 1
    //
    TEST(impl -> _OB_getRef() == 1);
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;

    ORB_var orb;
    try
    {
        orb = ORB_init(argc, argv);

        POA_var root = TestUtil_GetRootPOA(orb);

	printf("Testing collocated reference-counted servant... ");
	fflush(stdout);
        TestRefCount(orb, root);
	printf("OK!\n");
	fflush(stdout);
    }
    catch(const Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
        status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
