// **********************************************************************
//
// Copyright (c) 2000
// Object Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OBE/CORBA.h>

#include <TestMacro.h>
#include <Test_skel.h>
#include <TestRef_impl.h>
#include <TestServer_impl.h>
#include <TestUtil.h>

#include <errno.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

using namespace CORBA;
using namespace PortableServer;

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);

        POA_var root = TestUtil_GetRootPOA(orb);
	POAManager_var manager = root -> the_POAManager();

        //
        // Create servants
        //
        TestRef_impl* impl1 = new TestRef_impl();
        ServantBase_var serv1 = impl1;
        TestRef_impl* impl2 = new TestRef_impl();
        ServantBase_var serv2 = impl2;

        //
        // Activate servants
        //
        ObjectId_var id1 = root -> activate_object(impl1);
        ObjectId_var id2 = root -> activate_object(impl2);

        //
        // Create info
        //
        Object_var obj;
        TestInfoSeq info;
        info.length(2);
        obj = root -> servant_to_reference(impl1);
        info[0].obj = Test::_narrow(obj);
        info[0].except_id = string_dup("");
        obj = root -> servant_to_reference(impl2);
        info[1].obj = Test::_narrow(obj);
        info[1].except_id = string_dup("");

	//
	// Create server
	//
        TestServer_impl* serverImpl = new TestServer_impl(orb, info);
	ServantBase_var serv = serverImpl;
	ObjectId_var serverId = root -> activate_object(serverImpl);
	obj = root -> servant_to_reference(serverImpl);
	String_var s = orb -> object_to_string(obj);

        //
        // Save reference
        //
        const char* refFile = "Test.ref";
	FILE* f = fopen(refFile, "w");
	if(f == 0)
	{
	    fprintf(stderr, "can't open `%s': %s\n", refFile, strerror(errno));
	    return EXIT_FAILURE;
	}
	fprintf(f, "%s\n", s.in());
	fclose(f);

        //
        // Run implementation
        //
        manager -> activate();
        orb -> run();

	//
	// Deactivate each object
	//
	root -> deactivate_object(serverId);
	root -> deactivate_object(id2);
	root -> deactivate_object(id1);

	//
	// The servant's ref count now should be 1
	//
	TEST(impl1 -> _OB_getRef() == 1);
	TEST(impl2 -> _OB_getRef() == 1);
	TEST(serverImpl -> _OB_getRef() == 1);

#ifdef HAVE__UNLINK
        _unlink(refFile);
#else
        unlink(refFile);
#endif
    }
    catch(const Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
        status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
